# -*- coding: utf-8 -*-

__revision__ = '$Id: languagenames.py 8725 2006-12-17 22:39:07Z luks $'

languageNames = {
	u'ART': u'Artificial (Other)',
	u'ROH': u'Raeto-Romance',
	u'SCO': u'Scots',
	u'SCN': u'Sicilian',
	u'ROM': u'Romany',
	u'RON': u'Romanian',
	u'OSS': u'Ossetian; Ossetic',
	u'ALE': u'Aleut',
	u'MNI': u'Manipuri',
	u'NWC': u'Classical Newari; Old Newari; Classical Nepal Bhasa',
	u'OSA': u'Osage',
	u'MNC': u'Manchu',
	u'MWR': u'Marwari',
	u'VEN': u'Venda',
	u'MWL': u'Mirandese',
	u'FAS': u'Persian',
	u'FAT': u'Fanti',
	u'FAN': u'Fang',
	u'FAO': u'Faroese',
	u'DIN': u'Dinka',
	u'HYE': u'Armenian',
	u'DSB': u'Lower Sorbian',
	u'CAR': u'Carib',
	u'DIV': u'Divehi',
	u'TEL': u'Telugu',
	u'TEM': u'Timne',
	u'NBL': u'Ndebele, South; South Ndebele',
	u'TER': u'Tereno',
	u'TET': u'Tetum',
	u'SUN': u'Sundanese',
	u'KUT': u'Kutenai',
	u'SUK': u'Sukuma',
	u'KUR': u'Kurdish',
	u'KUM': u'Kumyk',
	u'SUS': u'Susu',
	u'NEW': u'Newari; Nepal Bhasa',
	u'KUA': u'Kuanyama; Kwanyama',
	u'MEN': u'Mende',
	u'LEZ': u'Lezghian',
	u'GLA': u'Gaelic; Scottish Gaelic',
	u'BOS': u'Bosnian',
	u'GLE': u'Irish',
	u'EKA': u'Ekajuk',
	u'GLG': u'Gallegan',
	u'AKA': u'Akan',
	u'BOD': u'Tibetan',
	u'GLV': u'Manx',
	u'JRB': u'Judeo-Arabic',
	u'VIE': u'Vietnamese',
	u'IPK': u'Inupiaq',
	u'UZB': u'Uzbek',
	u'BRE': u'Breton',
	u'BRA': u'Braj',
	u'AYM': u'Aymara',
	u'CHA': u'Chamorro',
	u'CHB': u'Chibcha',
	u'CHE': u'Chechen',
	u'CHG': u'Chagatai',
	u'CHK': u'Chuukese',
	u'CHM': u'Mari',
	u'CHN': u'Chinook jargon',
	u'CHO': u'Choctaw',
	u'CHP': u'Chipewyan',
	u'CHR': u'Cherokee',
	u'CHU': u'Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic',
	u'CHV': u'Chuvash',
	u'CHY': u'Cheyenne',
	u'MSA': u'Malay',
	u'III': u'Sichuan Yi',
	u'ACE': u'Achinese',
	u'IBO': u'Igbo',
	u'IBA': u'Iban',
	u'XHO': u'Xhosa',
	u'DEU': u'German',
	u'CAT': u'Catalan; Valencian',
	u'DEL': u'Delaware',
	u'DEN': u'Slave (Athapascan)',
	u'CAD': u'Caddo',
	u'TAT': u'Tatar',
	u'RAJ': u'Rajasthani',
	u'SPA': u'Spanish; Castilian',
	u'TAM': u'Tamil',
	u'TAH': u'Tahitian',
	u'AFH': u'Afrihili',
	u'ENG': u'English',
	u'CSB': u'Kashubian',
	u'NYN': u'Nyankole',
	u'NYO': u'Nyoro',
	u'SID': u'Sidamo',
	u'NYA': u'Chichewa; Chewa; Nyanja',
	u'SIN': u'Sinhala; Sinhalese',
	u'AFR': u'Afrikaans',
	u'LAM': u'Lamba',
	u'SND': u'Sindhi',
	u'MAR': u'Marathi',
	u'LAH': u'Lahnda',
	u'NYM': u'Nyamwezi',
	u'SNA': u'Shona',
	u'LAD': u'Ladino',
	u'SNK': u'Soninke',
	u'MAD': u'Madurese',
	u'MAG': u'Magahi',
	u'MAI': u'Maithili',
	u'MAH': u'Marshallese',
	u'LAV': u'Latvian',
	u'MAL': u'Malayalam',
	u'MAN': u'Mandingo',
	u'ZND': u'Zande',
	u'ZEN': u'Zenaga',
	u'KBD': u'Kabardian',
	u'ITA': u'Italian',
	u'VAI': u'Vai',
	u'TSN': u'Tswana',
	u'TSO': u'Tsonga',
	u'TSI': u'Tsimshian',
	u'BYN': u'Blin; Bilin',
	u'FIJ': u'Fijian',
	u'FIN': u'Finnish',
	u'EUS': u'Basque',
	u'CEB': u'Cebuano',
	u'DAN': u'Danish',
	u'NOG': u'Nogai',
	u'NOB': u'Norwegian Bokmål; Bokmål, Norwegian',
	u'DAK': u'Dakota',
	u'CES': u'Czech',
	u'DAR': u'Dargwa',
	u'DAY': u'Dayak',
	u'NOR': u'Norwegian',
	u'KPE': u'Kpelle',
	u'GUJ': u'Gujarati',
	u'MDF': u'Moksha',
	u'MAS': u'Masai',
	u'LAO': u'Lao',
	u'MDR': u'Mandar',
	u'GON': u'Gondi',
	u'SMS': u'Skolt Sami',
	u'SMO': u'Samoan',
	u'SMN': u'Inari Sami',
	u'SMJ': u'Lule Sami',
	u'GOT': u'Gothic',
	u'SME': u'Northern Sami',
	u'BLA': u'Siksika',
	u'SMA': u'Southern Sami',
	u'GOR': u'Gorontalo',
	u'AST': u'Asturian; Bable',
	u'ORM': u'Oromo',
	u'QUE': u'Quechua',
	u'ORI': u'Oriya',
	u'CRH': u'Crimean Tatar; Crimean Turkish',
	u'ASM': u'Assamese',
	u'PUS': u'Pushto',
	u'DGR': u'Dogrib',
	u'LTZ': u'Luxembourgish; Letzeburgesch',
	u'NDO': u'Ndonga',
	u'GEZ': u'Geez',
	u'ISL': u'Icelandic',
	u'LAT': u'Latin',
	u'MAK': u'Makasar',
	u'ZAP': u'Zapotec',
	u'YID': u'Yiddish',
	u'KOK': u'Konkani',
	u'KOM': u'Komi',
	u'KON': u'Kongo',
	u'UKR': u'Ukrainian',
	u'TON': u'Tonga (Tonga Islands)',
	u'KOS': u'Kosraean',
	u'KOR': u'Korean',
	u'TOG': u'Tonga (Nyasa)',
	u'HUN': u'Hungarian',
	u'HUP': u'Hupa',
	u'CYM': u'Welsh',
	u'UDM': u'Udmurt',
	u'BEJ': u'Beja',
	u'BEN': u'Bengali',
	u'BEL': u'Belarusian',
	u'BEM': u'Bemba',
	u'AAR': u'Afar',
	u'NZI': u'Nzima',
	u'SAH': u'Yakut',
	u'SAN': u'Sanskrit',
	u'SAM': u'Samaritan Aramaic',
	u'SAG': u'Sango',
	u'SAD': u'Sandawe',
	u'RAR': u'Rarotongan',
	u'RAP': u'Rapanui',
	u'SAS': u'Sasak',
	u'SAT': u'Santali',
	u'MIN': u'Minangkabau',
	u'LIM': u'Limburgan; Limburger; Limburgish',
	u'LIN': u'Lingala',
	u'LIT': u'Lithuanian',
	u'EFI': u'Efik',
	u'BTK': u'Batak (Indonesia)',
	u'KAC': u'Kachin',
	u'KAB': u'Kabyle',
	u'KAA': u'Kara-Kalpak',
	u'KAN': u'Kannada',
	u'KAM': u'Kamba',
	u'KAL': u'Kalaallisut; Greenlandic',
	u'KAS': u'Kashmiri',
	u'KAR': u'Karen',
	u'KAU': u'Kanuri',
	u'KAT': u'Georgian',
	u'KAZ': u'Kazakh',
	u'TYV': u'Tuvinian',
	u'AWA': u'Awadhi',
	u'URD': u'Urdu',
	u'DOI': u'Dogri',
	u'TPI': u'Tok Pisin',
	u'MRI': u'Maori',
	u'ABK': u'Abkhazian',
	u'TKL': u'Tokelau',
	u'NLD': u'Dutch; Flemish',
	u'OJI': u'Ojibwa',
	u'OCI': u'Occitan (post 1500); Provençal',
	u'WOL': u'Wolof',
	u'JAV': u'Javanese',
	u'HRV': u'Croatian',
	u'DYU': u'Dyula',
	u'SSW': u'Swati',
	u'MUL': u'Multiple languages',
	u'HIL': u'Hiligaynon',
	u'HIM': u'Himachali',
	u'HIN': u'Hindi',
	u'BAS': u'Basa',
	u'GBA': u'Gbaya',
	u'WLN': u'Walloon',
	u'BAD': u'Banda',
	u'NEP': u'Nepali',
	u'CRE': u'Cree',
	u'BAN': u'Balinese',
	u'BAL': u'Baluchi',
	u'BAM': u'Bambara',
	u'BAK': u'Bashkir',
	u'SHN': u'Shan',
	u'ARP': u'Arapaho',
	u'ARW': u'Arawak',
	u'ARA': u'Arabic',
	u'ARC': u'Aramaic',
	u'ARG': u'Aragonese',
	u'SEL': u'Selkup',
	u'ARN': u'Araucanian',
	u'LUS': u'Lushai',
	u'MUS': u'Creek',
	u'LUA': u'Luba-Lulua',
	u'LUB': u'Luba-Katanga',
	u'LUG': u'Ganda',
	u'LUI': u'Luiseno',
	u'LUN': u'Lunda',
	u'LUO': u'Luo (Kenya and Tanzania)',
	u'IKU': u'Inuktitut',
	u'TUR': u'Turkish',
	u'TUK': u'Turkmen',
	u'TUM': u'Tumbuka',
	u'COP': u'Coptic',
	u'COS': u'Corsican',
	u'COR': u'Cornish',
	u'ILO': u'Iloko',
	u'GWI': u'Gwich´in',
	u'TLI': u'Tlingit',
	u'TLH': u'Klingon; tlhIngan-Hol',
	u'POR': u'Portuguese',
	u'PON': u'Pohnpeian',
	u'POL': u'Polish',
	u'TGK': u'Tajik',
	u'TGL': u'Tagalog',
	u'FRA': u'French',
	u'BHO': u'Bhojpuri',
	u'SWA': u'Swahili',
	u'DUA': u'Duala',
	u'SWE': u'Swedish',
	u'YAP': u'Yapese',
	u'TIV': u'Tiv',
	u'YAO': u'Yao',
	u'XAL': u'Kalmyk',
	u'FRY': u'Frisian',
	u'GAY': u'Gayo',
	u'OTA': u'Turkish, Ottoman (1500-1928)',
	u'HMN': u'Hmong',
	u'HMO': u'Hiri Motu',
	u'GAA': u'Ga',
	u'FUR': u'Friulian',
	u'MLG': u'Malagasy',
	u'SLV': u'Slovenian',
	u'FIL': u'Filipino; Pilipino',
	u'MLT': u'Maltese',
	u'SLK': u'Slovak',
	u'FUL': u'Fulah',
	u'JPN': u'Japanese',
	u'VOL': u'Volapük',
	u'VOT': u'Votic',
	u'IND': u'Indonesian',
	u'AVE': u'Avestan',
	u'JPR': u'Judeo-Persian',
	u'AVA': u'Avaric',
	u'PAP': u'Papiamento',
	u'EWO': u'Ewondo',
	u'PAU': u'Palauan',
	u'EWE': u'Ewe',
	u'PAG': u'Pangasinan',
	u'PAM': u'Pampanga',
	u'PAN': u'Panjabi; Punjabi',
	u'KIR': u'Kirghiz',
	u'NIA': u'Nias',
	u'KIK': u'Kikuyu; Gikuyu',
	u'SYR': u'Syriac',
	u'KIN': u'Kinyarwanda',
	u'NIU': u'Niuean',
	u'EPO': u'Esperanto',
	u'JBO': u'Lojban',
	u'MIC': u'Mi\'kmaq; Micmac',
	u'THA': u'Thai',
	u'HAI': u'Haida',
	u'ELL': u'Greek, Modern (1453-)',
	u'ADY': u'Adyghe; Adygei',
	u'ELX': u'Elamite',
	u'ADA': u'Adangme',
	u'GRB': u'Grebo',
	u'HAT': u'Haitian; Haitian Creole',
	u'HAU': u'Hausa',
	u'HAW': u'Hawaiian',
	u'BIN': u'Bini',
	u'AMH': u'Amharic',
	u'BIK': u'Bikol',
	u'BIH': u'Bihari',
	u'MOS': u'Mossi',
	u'MOH': u'Mohawk',
	u'MON': u'Mongolian',
	u'MOL': u'Moldavian',
	u'BIS': u'Bislama',
	u'TVL': u'Tuvalu',
	u'IJO': u'Ijo',
	u'EST': u'Estonian',
	u'KMB': u'Kimbundu',
	u'UMB': u'Umbundu',
	u'TMH': u'Tamashek',
	u'FON': u'Fon',
	u'HSB': u'Upper Sorbian',
	u'RUN': u'Rundi',
	u'RUS': u'Russian',
	u'PLI': u'Pali',
	u'SRD': u'Sardinian',
	u'ACH': u'Acoli',
	u'NDE': u'Ndebele, North; North Ndebele',
	u'DZO': u'Dzongkha',
	u'KRU': u'Kurukh',
	u'SRR': u'Serer',
	u'IDO': u'Ido',
	u'SRP': u'Serbian',
	u'KRO': u'Kru',
	u'KRC': u'Karachay-Balkar',
	u'NDS': u'Low German; Low Saxon; German, Low; Saxon, Low',
	u'ZUN': u'Zuni',
	u'ZUL': u'Zulu',
	u'TWI': u'Twi',
	u'NSO': u'Northern Sotho, Pedi; Sepedi',
	u'SOM': u'Somali',
	u'SON': u'Songhai',
	u'SOT': u'Sotho, Southern',
	u'MKD': u'Macedonian',
	u'HER': u'Herero',
	u'LOL': u'Mongo',
	u'HEB': u'Hebrew',
	u'LOZ': u'Lozi',
	u'GIL': u'Gilbertese',
	u'WAS': u'Washo',
	u'WAR': u'Waray',
	u'BUL': u'Bulgarian',
	u'WAL': u'Walamo',
	u'BUA': u'Buriat',
	u'BUG': u'Buginese',
	u'AZE': u'Azerbaijani',
	u'ZHA': u'Zhuang; Chuang',
	u'ZHO': u'Chinese',
	u'NNO': u'Norwegian Nynorsk; Nynorsk, Norwegian',
	u'UIG': u'Uighur; Uyghur',
	u'MYV': u'Erzya',
	u'INH': u'Ingush',
	u'KHM': u'Khmer',
	u'MYA': u'Burmese',
	u'KHA': u'Khasi',
	u'INA': u'Interlingua (International Auxiliary Language Association)',
	u'NAH': u'Nahuatl',
	u'TIR': u'Tigrinya',
	u'NAP': u'Neapolitan',
	u'NAV': u'Navajo; Navaho',
	u'NAU': u'Nauru',
	u'GRN': u'Guarani',
	u'TIG': u'Tigre',
	u'YOR': u'Yoruba',
	u'ILE': u'Interlingue',
	u'SQI': u'Albanian',
}

# EOF



#! /usr/bin/env python
#
# Retrieve an artist by ID and display all official albums.
#
# Usage:
#	python getartist.py artist-id
#
# $Id: getartist.py 11853 2009-07-21 09:26:50Z luks $
#
import sys
import logging
import musicbrainz2.webservice as ws
import musicbrainz2.model as m

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


if len(sys.argv) < 2:
	print "Usage: getartist.py artist-id"
	sys.exit(1)

q = ws.Query()

try:
	# The result should include all official albums.
	#
	inc = ws.ArtistIncludes(
		releases=(m.Release.TYPE_OFFICIAL, m.Release.TYPE_ALBUM),
		tags=True, releaseGroups=True)
	artist = q.getArtistById(sys.argv[1], inc)
except ws.WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


print "Id         :", artist.id
print "Name       :", artist.name
print "SortName   :", artist.sortName
print "UniqueName :", artist.getUniqueName()
print "Type       :", artist.type
print "BeginDate  :", artist.beginDate
print "EndDate    :", artist.endDate
print "Tags       :", ', '.join([t.value for t in artist.tags])
print

if len(artist.getReleases()) == 0:
	print "No releases found."
else:
	print "Releases:"

for release in artist.getReleases():
	print
	print "Id        :", release.id
	print "Title     :", release.title
	print "ASIN      :", release.asin
	print "Text      :", release.textLanguage, '/', release.textScript
	print "Types     :", release.types

print

if len(artist.getReleaseGroups()) == 0:
	print
	print "No release groups found."
else:
	print
	print "Release groups:"

for rg in artist.getReleaseGroups():
	print
	print "Id        :", rg.id
	print "Title     :", rg.title
	print "Type      :", rg.type

print

#
# Using the release IDs and Query.getReleaseById(), you could now request 
# those releases, including the tracks, release events, the associated
# DiscIDs, and more. The 'getrelease.py' example shows how this works.
#

# EOF



#! /usr/bin/env python
#
# Search for a release group by name.
#
# Usage:
#	python findreleasegroup.py 'album name/Lucene query'
#
# Examples:
#	findreleasegroup.py "Signal morning"
#	findreleasegroup.py '"Gimme Fiction" AND artist:"Spoon"'
#	findreleasegroup.py '"Skylarking" AND type:"Album"'
#
import sys
import logging
from musicbrainz2.webservice import Query, ReleaseGroupFilter, WebServiceError

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
	

if len(sys.argv) < 2:
	print "Usage: findreleasegroup.py 'release group name/Lucene query'"
	sys.exit(1)

q = Query()

try:
	# Search for all release groups matching the given query. Limit the results
	# to the 5 best matches. The offset parameter could be used to page
	# through the results.
	#
	f = ReleaseGroupFilter(query=sys.argv[1], limit=5)
	results = q.getReleaseGroups(f)
except WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


# No error occurred, so display the results of the search. It consists of
# ReleaseGroupResult objects, where each contains an artist.
#
for result in results:
	releaseGroup = result.releaseGroup
	print "Score\t\t :", result.score
	print "Id        :", releaseGroup.id
	print "Name      :", releaseGroup.title
	print "Artist    :", releaseGroup.artist.name
	print "Type      :", releaseGroup.type
	print

#
# Now that you have release group IDs, you can request a release group in more detail, for
# example to display all official releases in that group. See the 'getreleasegroup.py'
# example on how achieve that.
#

# EOF



"""Tests for parsing artists using MbXmlParser."""
import unittest
from musicbrainz2.wsxml import MbXmlParser, ParseError
from musicbrainz2.model import NS_MMD_1
import StringIO
import os.path

VALID_DATA_DIR = os.path.join('test-data', 'valid')
INVALID_DATA_DIR = os.path.join('test-data', 'invalid')

VALID_ARTIST_DIR = os.path.join(VALID_DATA_DIR, 'artist')

def makeId(relativeUri, resType='artist'):
	return 'http://musicbrainz.org/%s/%s' % (resType, relativeUri)

class ParseArtistTest(unittest.TestCase):

	def __init__(self, name):
		unittest.TestCase.__init__(self, name)


	def testArtistBasic(self):
		f = os.path.join(VALID_ARTIST_DIR, 'Tori_Amos_1.xml')
		md = MbXmlParser().parse(f)
		artist = md.getArtist()

		self.failIf( artist is None )
		self.assertEquals(artist.getName(), 'Tori Amos')
		self.assertEquals(artist.getSortName(), 'Amos, Tori')
		self.assertEquals(artist.getBeginDate(), '1963-08-22')
		self.assertEquals(len(artist.getReleases()), 0)


	def testArtistFull(self):
		f = os.path.join(VALID_ARTIST_DIR, 'Tori_Amos_2.xml')
		md = MbXmlParser().parse(f)
		artist = md.getArtist()

		self.failIf( artist is None )
		self.assertEquals(artist.getId(),
			makeId('c0b2500e-0cef-4130-869d-732b23ed9df5'))
		self.assertEquals(artist.getName(), 'Tori Amos')
		self.assertEquals(artist.getSortName(), 'Amos, Tori')
		self.assert_(artist.getDisambiguation() is None)
		self.assertEquals(artist.getUniqueName(), artist.getName())
		self.assertEquals(artist.getBeginDate(), '1963-08-22')
		self.assertEquals(len(artist.getReleases()), 3)
		self.assertEquals(len(artist.getReleaseGroups()), 3)

		release1 = artist.getReleases()[0]
		self.assertEquals(release1.getTitle(), 'Strange Little Girls')
		self.assertEquals(release1.getAsin(), 'B00005NKYQ')

		# Check last release in more detail.
		#
		release3 = artist.getReleases()[2]
		self.assertEquals(release3.getId(),
			makeId('290e10c5-7efc-4f60-ba2c-0dfc0208fbf5', 'release'))
		self.assertEquals(len(release3.getTypes()), 2)
		self.assert_(NS_MMD_1 + 'Album' in release3.getTypes())
		self.assert_(NS_MMD_1 + 'Official' in release3.getTypes())
		self.assertEquals(release3.getTitle(), 'Under the Pink')
		self.assertEquals(release3.getAsin(), 'B000002IXU')
		self.assertEquals(release3.getArtist().getId(),
			makeId('c0b2500e-0cef-4130-869d-732b23ed9df5'))
		self.failIf(release3.getReleaseGroup() is None)
		self.assertEquals(release3.getReleaseGroup().id[-36:],
			'ef2b891f-ca73-3e14-b38b-a68699dab8c4')

		events = release3.getReleaseEvents()
		self.assertEquals(len(events), 5)
		self.assertEquals(events[0].getCountry(), 'DE')
		self.assertEquals(events[0].getDate(), '1994-01-28')
		self.assertEquals(events[4].getCountry(), 'AU')
		self.assertEquals(events[4].getDate(), '1994-11')

		self.assertEquals(release3.getEarliestReleaseDate(), '1994-01-28')

		#self.assertEquals(release3.getDiscCount(), 4)


	def testAliases(self):
		f = os.path.join(VALID_ARTIST_DIR, 'Tori_Amos_4.xml')
		md = MbXmlParser().parse(f)
		artist = md.getArtist()
		
		self.failIf( artist is None )
		self.assertEquals(artist.getDisambiguation(), 'yes, that one')
		self.assertEquals(artist.getName(), 'Tori Amos')
		self.assertEquals(artist.getUniqueName(),
			'Tori Amos (yes, that one)')

		aliases = artist.getAliases()
		self.assertEquals(len(aliases), 3)
		self.assertEquals(aliases[0].getValue(), 'Myra Ellen Amos')
		self.assertEquals(aliases[0].getScript(), 'Latn')
		self.assertEquals(aliases[1].getValue(), 'Myra Amos')
		self.assertEquals(aliases[2].getValue(), 'Torie Amos')
		self.assertEquals(aliases[2].getType(), NS_MMD_1 + 'Misspelling')


	def testTags(self):
		f = os.path.join(VALID_ARTIST_DIR, 'Tchaikovsky-2.xml')
		md = MbXmlParser().parse(f)
		artist = md.getArtist()
		
		self.failIf( artist is None )
		self.assertEquals(artist.getTag('classical').count, 100)
		self.assertEquals(artist.getTag('russian').count, 60)
		self.assertEquals(artist.getTag('romantic era').count, 40)
		self.assertEquals(artist.getTag('composer').count, 120)
		
	
	def testReleaseGroups(self):
		f = os.path.join(VALID_ARTIST_DIR, 'Tori_Amos_2.xml')
		md = MbXmlParser().parse(f)
		artist = md.getArtist()

		self.failIf(artist is None)
		releaseGroups = artist.getReleaseGroups()
		self.failIf(releaseGroups is None)
		self.assertEquals(len(releaseGroups), 3)

		expectedEntries = {
			'ef2b891f-ca73-3e14-b38b-a68699dab8c4': 'Under the Pink',
			'1fd43909-8056-3805-b2f9-c663ce7e71e6': 'To Venus and Back',
			'a69a1574-dfe3-3e2a-b499-d26d5e916041': 'Strange Little Girls'}

		for releaseGroup in releaseGroups:
			self.assertEquals(releaseGroup.getType(), NS_MMD_1 + 'Album')
			releaseGroupId = releaseGroup.id[-36:]
			self.assert_(releaseGroupId in expectedEntries)
			self.assertEquals(releaseGroup.getTitle(), expectedEntries[releaseGroupId])
			del expectedEntries[releaseGroupId]


	def testSearchResults(self):
		f = os.path.join(VALID_ARTIST_DIR, 'search_result_1.xml')
		md = MbXmlParser().parse(f)

		self.assertEquals(md.artistResultsOffset, 0)
		self.assertEquals(md.artistResultsCount, 47)

		results = md.artistResults
		self.assertEquals(len(results), 3)

		self.assertEquals(results[0].score, 100)
		artist1 = results[0].artist
		self.assertEquals(artist1.name, 'Tori Amos')

# EOF



"""Tests for parsing artists using MbXmlParser."""
import unittest
from musicbrainz2.wsxml import MbXmlParser, ParseError
from musicbrainz2.model import NS_MMD_1
import StringIO
import os.path

VALID_DATA_DIR = os.path.join('test-data', 'valid')
INVALID_DATA_DIR = os.path.join('test-data', 'invalid')

VALID_LABEL_DIR = os.path.join(VALID_DATA_DIR, 'label')

def makeId(relativeUri, resType='label'):
	return 'http://musicbrainz.org/%s/%s' % (resType, relativeUri)


class ParseLabelTest(unittest.TestCase):

	def __init__(self, name):
		unittest.TestCase.__init__(self, name)


	def testLabelBasic(self):
		f = os.path.join(VALID_LABEL_DIR, 'Atlantic_Records_1.xml')
		md = MbXmlParser().parse(f)
		label = md.getLabel()

		self.failIf( label is None )
		self.assertEquals(label.id,
			makeId('50c384a2-0b44-401b-b893-8181173339c7'))
		self.assertEquals(label.type, NS_MMD_1 + 'OriginalProduction')
		self.assertEquals(label.name, 'Atlantic Records')
		self.assertEquals(label.beginDate, '1947')
		self.assertEquals(label.endDate, None)
		self.assertEquals(label.country, 'US')
		self.assertEquals(label.code, '121')


	def testIncomplete(self):
		f = os.path.join(VALID_LABEL_DIR, 'Atlantic_Records_3.xml')
		md = MbXmlParser().parse(f)
		label = md.getLabel()

		self.failIf( label is None )
		self.assertEquals(label.id,
			makeId('50c384a2-0b44-401b-b893-8181173339c7'))
		self.assertEquals(label.code, None)


	def testLabelSubElements(self):
		f = os.path.join(VALID_LABEL_DIR, 'Atlantic_Records_2.xml')
		md = MbXmlParser().parse(f)
		label = md.getLabel()

		self.failIf( label is None )
		self.assertEquals(label.type, NS_MMD_1 + 'Distributor')
		self.assertEquals(label.name, 'Atlantic Records')
		self.assertEquals(label.sortName, 'AR SortName')
		self.assertEquals(label.disambiguation, 'fake')
		self.assertEquals(label.beginDate, '1947')
		self.assertEquals(label.endDate, '2047')
		self.assertEquals(label.country, 'US')
		self.assertEquals(label.code, '121')
		self.assertEquals(len(label.aliases), 1)

		alias = label.aliases[0]
		self.assertEquals(alias.value, 'Atlantic Rec.')

		self.assertEquals(label.getUniqueName(),
			'Atlantic Records (fake)')


	def testTags(self):
		f = os.path.join(VALID_LABEL_DIR, 'Atlantic_Records_3.xml')
		md = MbXmlParser().parse(f)
		label = md.getLabel()
		
		self.failIf( label is None )
		self.assertEquals(label.getTag('american').count, None)
		self.assertEquals(label.getTag('jazz').count, None)
		self.assertEquals(label.getTag('blues').count, None)


	def testSearchResults(self):
		f = os.path.join(VALID_LABEL_DIR, 'search_result_1.xml')
		md = MbXmlParser().parse(f)

		self.assertEquals(md.labelResultsOffset, 0)
		self.assertEquals(md.labelResultsCount, 2)
		self.assertEquals(md.getLabelResultsOffset(), 0)
		self.assertEquals(md.getLabelResultsCount(), 2)

		results = md.labelResults
		self.assertEquals(len(results), 2)

		self.assertEquals(results[0].score, 100)
		label1 = results[0].label
		self.assertEquals(label1.name, 'Atlantic Records')

		self.assertEquals(results[1].score, 46)
		label2 = results[1].label
		self.assertEquals(label2.name, 'DRO Atlantic')

# EOF



"""Tests for parsing tracks using MbXmlParser."""
import unittest
from musicbrainz2.wsxml import MbXmlParser, ParseError
from musicbrainz2.model import NS_MMD_1, NS_REL_1, Relation
import StringIO
import os.path

VALID_DATA_DIR = os.path.join('test-data', 'valid')
INVALID_DATA_DIR = os.path.join('test-data', 'invalid')

VALID_TRACK_DIR = os.path.join(VALID_DATA_DIR, 'track')

def makeId(relativeUri, resType='track'):
	return 'http://musicbrainz.org/%s/%s' % (resType, relativeUri)


class ParseTrackTest(unittest.TestCase):

	def __init__(self, name):
		unittest.TestCase.__init__(self, name)


	def testTrackBasic(self):
		f = os.path.join(VALID_TRACK_DIR, 'Silent_All_These_Years_1.xml')
		md = MbXmlParser().parse(f)
		track = md.getTrack()

		self.failIf( track is None )
		self.assertEquals(track.getTitle(), 'Silent All These Years')
		self.assertEquals(track.getDuration(), 253466)


	def testTrackRelations(self):
		f = os.path.join(VALID_TRACK_DIR, 'Silent_All_These_Years_2.xml')
		md = MbXmlParser().parse(f)
		track = md.getTrack()

		self.failIf( track is None )
		self.assertEquals(track.getTitle(), 'Silent All These Years')
		self.assertEquals(track.getDuration(), 253466)

		trackRels = track.getRelations(Relation.TO_TRACK)
		self.assertEquals(len(trackRels), 1)
		rel1 = trackRels[0]
		self.assertEquals(rel1.getType(), NS_REL_1 + 'Cover')
		self.assertEquals(rel1.getDirection(), Relation.DIR_BACKWARD)
		self.assertEquals(rel1.getTargetId(),
			makeId('31e1c0c4-967f-435e-b09a-35ee079ee234', 'track'))
		self.assert_( rel1.getBeginDate() is None )
		self.assert_( rel1.getEndDate() is None )

		self.failIf( rel1.getTarget() is None )
		self.assertEquals(rel1.getTarget().getId(),
			makeId('31e1c0c4-967f-435e-b09a-35ee079ee234'))
		self.assertEquals(rel1.getTarget().getArtist().getId(),
			makeId('5bcd4eaa-fae7-465f-9f03-d005b959ed02', 'artist'))


	def testTrackFull(self):
		f = os.path.join(VALID_TRACK_DIR, 'Silent_All_These_Years_4.xml')
		md = MbXmlParser().parse(f)
		track = md.getTrack()

		self.failIf( track is None )
		self.assertEquals(track.getTitle(), 'Silent All These Years')
		self.assertEquals(track.getDuration(), 253466)

		artist = track.getArtist()
		self.failIf( artist is None )
		self.assertEquals(artist.getId(),
			makeId('c0b2500e-0cef-4130-869d-732b23ed9df5', 'artist'))
		self.assertEquals(artist.getType(), NS_MMD_1 + 'Person')
		self.assertEquals(artist.getName(), 'Tori Amos')
		self.assertEquals(artist.getSortName(), 'Amos, Tori')

		puids = track.getPuids()
		self.assertEquals(len(puids), 7)
		self.assertEquals(puids[0], 'c2a2cee5-a8ca-4f89-a092-c3e1e65ab7e6')
		self.assertEquals(puids[6], '42ab76ea-5d42-4259-85d7-e7f2c69e4485')
		
		isrcs = track.isrcs
		self.assertEquals(len(isrcs), 1)
		self.assertEquals(isrcs[0], 'USPR37300012')

		releases = track.getReleases()
		self.assertEquals(len(releases), 1)
		self.assertEquals(releases[0].getTitle(), 'Little Earthquakes')
		self.assertEquals(releases[0].getTracksOffset(), 2)

	def testSearchResults(self):
		f = os.path.join(VALID_TRACK_DIR, 'search_result_1.xml')
		md = MbXmlParser().parse(f)

		self.assertEquals(md.trackResultsOffset, 7)
		self.assertEquals(md.trackResultsCount, 100)

		results = md.getTrackResults()
		self.assertEquals(len(results), 3)

		self.assertEquals(results[0].getScore(), 100)
		track1 = results[0].getTrack()
		self.assertEquals(track1.getTitle(), 'Little Earthquakes')

# EOF



"""Tests for webservice.Query."""
import unittest
from musicbrainz2.model import Tag
from musicbrainz2.model import Rating
from musicbrainz2.model import Release
from musicbrainz2.webservice import Query, IWebService, AuthenticationError, RequestError


class FakeWebService(IWebService):

	def __init__(self):
		self.data = []

	def post(self, entity, id_, data, version='1'):
		self.data.append((entity, id_, data, version))
		
class FakeBadAuthWebService(IWebService):
	def post(self, entity, id_, data, version='1'):
		raise AuthenticationError()

class FakeBadRequestWebService(IWebService):
	def post(self, entity, id_, data, version='1'):
		raise RequestError()

class QueryTest(unittest.TestCase):

	def testAddToUserCollection(self):
		ws = FakeWebService()
		q = Query(ws)

		r1 = "9e186398-9ae2-45bf-a9f6-d26bc350221e"
		r2 = "http://musicbrainz.org/release/6b050dcf-7ab1-456d-9e1b-c3c41c18eed2"
		r3 = Release("d3cc336e-1010-4252-9091-7923f0429824")

		q.addToUserCollection([r1, r2, r3])
		self.assertEquals(len(ws.data), 1)
		res = ws.data[0]
		self.assertEquals(res[0], "collection")
		self.assertEquals(res[2], "add=9e186398-9ae2-45bf-a9f6-d26bc350221e%2C6b050dcf-7ab1-456d-9e1b-c3c41c18eed2%2Cd3cc336e-1010-4252-9091-7923f0429824")

	def testRemoveFromUserCollection(self):
		ws = FakeWebService()
		q = Query(ws)

		r1 = "9e186398-9ae2-45bf-a9f6-d26bc350221e"
		r2 = "http://musicbrainz.org/release/6b050dcf-7ab1-456d-9e1b-c3c41c18eed2"
		r3 = Release("d3cc336e-1010-4252-9091-7923f0429824")

		q.removeFromUserCollection([r1, r2, r3])
		self.assertEquals(len(ws.data), 1)
		res = ws.data[0]
		self.assertEquals(res[0], "collection")
		self.assertEquals(res[2], "remove=9e186398-9ae2-45bf-a9f6-d26bc350221e%2C6b050dcf-7ab1-456d-9e1b-c3c41c18eed2%2Cd3cc336e-1010-4252-9091-7923f0429824")

	def testSubmitUserTags(self):
		ws = FakeWebService()
		q = Query(ws)
		t1 = [u"foo", u"bar", u"f\u014do"]
		t2 = [Tag(u"foo"), Tag(u"bar"), Tag(u"f\u014do")]

		prefix = 'http://musicbrainz.org/artist/'
		uri = prefix + 'c0b2500e-0cef-4130-869d-732b23ed9df5'

		q.submitUserTags(uri, t1)
		q.submitUserTags(uri, t2)

		self.assertEquals(len(ws.data), 2)
		self.assertEquals(ws.data[0], ws.data[1])

		q.submitUserRating(uri, Rating(5))
		q.submitUserRating(uri, 5)

		self.assertEquals(len(ws.data), 4)
		self.assertEquals(ws.data[2], ws.data[3])
		
	def testSubmitIsrc(self):
		tracks2isrcs = {
					    '6a47088b-d9e0-4088-868a-394ee3c6cd33':'NZABC0800001', 
					    'b547acbc-58c6-4a31-9806-e2348db3a167':'NZABC0800002'
		}
		ws = FakeWebService()
		q = Query(ws)
		
		q.submitISRCs(tracks2isrcs)
		
		self.assertEquals(len(ws.data), 1)
		req = ws.data[0]
		qstring = 'isrc=6a47088b-d9e0-4088-868a-394ee3c6cd33+NZABC0800001&isrc=b547acbc-58c6-4a31-9806-e2348db3a167+NZABC0800002'
		self.assertEquals(req[0], 'track')
		self.assertEquals(req[2], qstring)
		
	def testSubmitIsrcBadUser(self):
		ws = FakeBadAuthWebService()
		q = Query(ws)
		
		self.assertRaises(AuthenticationError, q.submitISRCs, {})
	
	def testSubmitIsrcBadTrack(self):
		ws = FakeBadRequestWebService()
		q = Query(ws)
		
		self.assertRaises(RequestError, q.submitISRCs, {})
		
	def testSubmitPuid(self):
		tracks2puids = {
					    '6a47088b-d9e0-4088-868a-394ee3c6cd33':'c2a2cee5-a8ca-4f89-a092-c3e1e65ab7e6', 
					    'b547acbc-58c6-4a31-9806-e2348db3a167':'c2a2cee5-a8ca-4f89-a092-c3e1e65ab7e6'
		}
		ws = FakeWebService()
		q = Query(ws, clientId='test-1')
		
		q.submitPuids(tracks2puids)
		
		self.assertEquals(len(ws.data), 1)
		req = ws.data[0]
		qstring = 'client=test-1&puid=6a47088b-d9e0-4088-868a-394ee3c6cd33+c2a2cee5-a8ca-4f89-a092-c3e1e65ab7e6&puid=b547acbc-58c6-4a31-9806-e2348db3a167+c2a2cee5-a8ca-4f89-a092-c3e1e65ab7e6'
		self.assertEquals(req[0], 'track')
		self.assertEquals(req[2], qstring)
		
	def testSubmitPuidBadUser(self):
		ws = FakeBadAuthWebService()
		q = Query(ws, clientId='test-1')
		
		self.assertRaises(AuthenticationError, q.submitPuids, {})

	def testSubmitPuidBadTrack(self):
		ws = FakeBadRequestWebService()
		q = Query(ws, clientId='test-1')
		
		self.assertRaises(RequestError, q.submitPuids, {})
		
	def testSubmitPuidNoClient(self):
		ws = FakeWebService()
		q = Query(ws)
		
		self.assertRaises(AssertionError, q.submitPuids, None)

	def testSubmitCDStubNoClient(self):
		ws = FakeWebService()
		q = Query(ws)
		
		self.assertRaises(AssertionError, q.submitCDStub, None)

	def testSubmitCdStub(self):
		from musicbrainz2.model import Disc, CDStub

		ws = FakeWebService()
		q = Query(ws, clientId='test-1')

		discid = "6EmGGSLhuDYz2lNXtqrCiCCqO0o-"
		disc = Disc(discid)
		disc.firstTrackNum = 1
		disc.lastTrackNum = 4
		disc.sectors = 89150
		disc.addTrack( (150, 20551) )
		disc.addTrack( (20701, 26074) )
		disc.addTrack( (46775, 19438) )
		disc.addTrack( (66213, 22937) )

		cdstub = CDStub(disc)
		cdstub.artist = "artist"
		cdstub.title = "title"
		cdstub.addTrack("trackname1")
		cdstub.addTrack("trackname2")
		cdstub.addTrack("trackname3")
		cdstub.addTrack("trackname4")
		q.submitCDStub(cdstub)

		cdstub.barcode = "12345"
		cdstub.comment = "acomment"
		q.submitCDStub(cdstub)

		self.assertEquals(len(ws.data), 2)
		req = ws.data[0]
		qstring = 'client=test-1&discid=6EmGGSLhuDYz2lNXtqrCiCCqO0o-&title=title&artist=artist&track0=trackname1&track1=trackname2&track2=trackname3&track3=trackname4&toc=1+4+89150+150+20701+46775+66213'
		self.assertEquals(req[0], 'release')
		self.assertEquals(req[2], qstring)

		req = ws.data[1]
		qstring = 'client=test-1&discid=6EmGGSLhuDYz2lNXtqrCiCCqO0o-&title=title&artist=artist&barcode=12345&comment=acomment&track0=trackname1&track1=trackname2&track2=trackname3&track3=trackname4&toc=1+4+89150+150+20701+46775+66213'
		self.assertEquals(req[2], qstring)

		cdstub._tracks = []
		cdstub.addTrack("tname1", "artist1")
		cdstub.addTrack("tname2", "artist2")
		cdstub.addTrack("tname3", "artist3")
		cdstub.addTrack("tname4", "artist4")
		q.submitCDStub(cdstub)
		self.assertEquals(len(ws.data), 3)
		req = ws.data[2]
		qstring = 'client=test-1&discid=6EmGGSLhuDYz2lNXtqrCiCCqO0o-&title=title&artist=artist&barcode=12345&comment=acomment&track0=tname1&artist0=artist1&track1=tname2&artist1=artist2&track2=tname3&artist2=artist3&track3=tname4&artist3=artist4&toc=1+4+89150+150+20701+46775+66213'
		self.assertEquals(req[2], qstring)




# EOF



"""Tests for parsing release groups using MbXmlParser."""
import unittest
from musicbrainz2.wsxml import MbXmlParser, MbXmlWriter, ParseError
from musicbrainz2.model import ReleaseGroup, NS_MMD_1
from StringIO import StringIO
import os.path

VALID_DATA_DIR = os.path.join('test-data', 'valid')
# Only have tests for valid data, ATM.

RELEASEGROUP_DIR = os.path.join(VALID_DATA_DIR, 'release-group')

def makeId(relativeUri):
	return 'http://musicbrainz.org/release-group/' + relativeUri

class ParseReleaseGroupTest(unittest.TestCase):

	def testReleaseGroupBasic(self):
		f = os.path.join(RELEASEGROUP_DIR, 'The_Cure_1.xml')
		md = MbXmlParser().parse(f)
		releaseGroup = md.getReleaseGroup()
		
		self.failIf(releaseGroup is None)
		self.assertEquals(releaseGroup.getId(),
			makeId('c6a62b78-70f7-44f7-b159-064f6b7ba03a'))
		self.assertEquals(releaseGroup.getTitle(), 'The Cure')
		self.assertEquals(releaseGroup.getType(), NS_MMD_1 + 'Album')


	def testReleaseGroupFull(self):
		f = os.path.join(RELEASEGROUP_DIR, 'The_Cure_1.xml')
		md = MbXmlParser().parse(f)
		releaseGroup = md.getReleaseGroup()

		self.failIf(releaseGroup is None)

		releases = releaseGroup.getReleases()
		self.failIf(releases is None)
		self.assertEquals(len(releases), 4)

		# Check releases, which are in no particular order.
		expectedIds = [
			'd984e1a3-7281-46bb-ad8b-1478a00f2fbf',
			'c100a398-3132-48a8-a5fc-c3e908ac17dc',
			'24bec892-b21d-47d8-a288-dc6450152574',
			'61a4ec51-fa34-4757-85d7-83231776ed14']
		actualIds = [release.id[-36:] for release in releases]
		for expectedId in expectedIds:
			self.assert_(expectedId in actualIds)

		# Check artist
		self.assertEquals(releaseGroup.getArtist().getName(), 'The Cure')
		self.assertEquals(releaseGroup.getArtist().id[-36:],
			'69ee3720-a7cb-4402-b48d-a02c366f2bcf')


	def testSearchResults(self):
		f = os.path.join(RELEASEGROUP_DIR, 'search_result_1.xml')
		md = MbXmlParser().parse(f)
		releaseGroups = md.getReleaseGroupResults()

		self.failIf(releaseGroups is None)
		self.assertEquals(md.getReleaseGroupResultsOffset(), 0)
		self.assertEquals(md.getReleaseGroupResultsCount(), 3)

		expectedEntries = {
			'963eac15-e3da-3a92-aa5c-2ec23bfb6ec2': ['Signal Morning', 100],
			'0bd324a3-1c90-3bdb-8ca4-4101a580c62c': ['Circulatory System', 98],
			'ea7d8352-7751-30be-8490-bb6df737f47c': ['Inside Views', 90]}
		for result in releaseGroups:
			releaseGroup = result.releaseGroup
			self.failIf(releaseGroup is None)

			releaseGroupId = releaseGroup.id[-36:]
			self.assert_(releaseGroupId in expectedEntries)

			expectedTitle, expectedScore = expectedEntries[releaseGroupId]
			self.assertEquals(releaseGroup.title, expectedTitle)
			self.assertEquals(result.score, expectedScore)

			del expectedEntries[releaseGroupId]


	def testSerialize(self):
		# Get initial.
		f = os.path.join(RELEASEGROUP_DIR, 'The_Cure_1.xml')
		md1 = MbXmlParser().parse(f)

		# Serialize.
		outbuffer = StringIO()
		MbXmlWriter().write(outbuffer, md1)

		# Deserialize.
		inbuffer = StringIO(outbuffer.getvalue().encode('utf-8'))
		md2 = MbXmlParser().parse(inbuffer)

		# Check
		releaseGroup = md2.getReleaseGroup()

		self.failIf(releaseGroup is None)
		self.assertEquals(len(releaseGroup.getReleases()), 4)
		self.assertEquals(releaseGroup.getId(),
			makeId('c6a62b78-70f7-44f7-b159-064f6b7ba03a'))
		self.assertEquals(releaseGroup.getTitle(), 'The Cure')
		self.assertEquals(releaseGroup.getType(), NS_MMD_1 + 'Album')

	def testEmptyType(self):
		f = os.path.join(RELEASEGROUP_DIR, 'The_Cure_1.xml')
		md1 = MbXmlParser().parse(f)

		releaseGroup1 = md1.getReleaseGroup()
		releaseGroup1.setType(None)

		# Serialize.
		outbuffer = StringIO()
		MbXmlWriter().write(outbuffer, md1)

		inbuffer = StringIO(outbuffer.getvalue().encode('utf-8'))
		md2 = MbXmlParser().parse(inbuffer)

		# Check
		releaseGroup2 = md2.getReleaseGroup()

		self.failIf(releaseGroup2 is None)
		self.assertEquals(releaseGroup2.getType(), None)



#! /usr/bin/env python
#
# This queries the tags a user has applied to an entity and submits a changed
# list of tags to the MusicBrainz server.
#
# Usage:
#    python tag.py
#
# $Id: folksonomytags.py 9714 2008-03-01 09:05:48Z matt $
#
import getpass
import sys
import logging
import musicbrainz2.webservice as mbws
from musicbrainz2.model import Tag
from musicbrainz2.utils import extractUuid

MB_HOST = 'test.musicbrainz.org'

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


# Get the username and password
username = raw_input('Username: ')
password = getpass.getpass('Password: ')

# Ask for a MBID to tag.
mbid = raw_input('Enter an absolute MB ID: ')

# Set the authentication for the webservice (only needed for tag submission).
service = mbws.WebService(host=MB_HOST, username=username, password=password)

# Create a new Query object which will provide
# us an interface to the MusicBrainz web service.
query = mbws.Query(service)

# Read and print the current tags for the given MBID
tags = query.getUserTags(mbid)
print
print 'Current tags: '
print ', '.join([tag.value for tag in tags])


# Ask the user for new tags and submit them
tag_str = raw_input('Enter new tags: ')
new_tags = [Tag(tag.strip()) for tag in tag_str.split(',')]

query.submitUserTags(mbid, new_tags)

print 'Tags submitted.'

# EOF



"""Tests for parsing releases using MbXmlParser."""
import unittest
from musicbrainz2.wsxml import MbXmlParser, ParseError
from musicbrainz2.model import VARIOUS_ARTISTS_ID, NS_MMD_1, \
	Relation, ReleaseEvent
import StringIO
import os.path

VALID_DATA_DIR = os.path.join('test-data', 'valid')
INVALID_DATA_DIR = os.path.join('test-data', 'invalid')

VALID_RELEASE_DIR = os.path.join(VALID_DATA_DIR, 'release')

def makeId(relativeUri):
	return 'http://musicbrainz.org/release/' + relativeUri

class ParseReleaseTest(unittest.TestCase):

	def __init__(self, name):
		unittest.TestCase.__init__(self, name)


	def testReleaseBasic(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Little_Earthquakes_1.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )
		self.assertEquals(release.getId(),
			makeId('02232360-337e-4a3f-ad20-6cdd4c34288c'))
		self.assertEquals(release.getTitle(), 'Little Earthquakes')
		self.assertEquals(release.getTextLanguage(), 'ENG')
		self.assertEquals(release.getTextScript(), 'Latn')

		self.assertEquals(len(release.getTypes()), 2)
		self.assert_(NS_MMD_1 + 'Album' in release.getTypes())
		self.assert_(NS_MMD_1 + 'Official' in release.getTypes())


	def testReleaseFull(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Little_Earthquakes_2.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )
		self.assertEquals(release.getId(),
			makeId('02232360-337e-4a3f-ad20-6cdd4c34288c'))

		self.assertEquals(release.getArtist().getName(), 'Tori Amos')

		events = release.getReleaseEventsAsDict()
		self.assertEquals(len(events), 3)
		self.assertEquals(events['GB'], '1992-01-13')
		self.assertEquals(events['DE'], '1992-01-17')
		self.assertEquals(events['US'], '1992-02-25')

		date = release.getEarliestReleaseDate()
		self.assertEquals(date, '1992-01-13')
		event = release.getEarliestReleaseEvent()
		self.assertEquals(event.date, date)
		self.assertEquals(event.country, 'GB')

		discs = release.getDiscs()
		self.assertEquals(len(discs), 3)
		self.assertEquals(discs[0].getId(), 'ILKp3.bZmvoMO7wSrq1cw7WatfA-')
		self.assertEquals(discs[1].getId(), 'ejdrdtX1ZyvCb0g6vfJejVaLIK8-')
		self.assertEquals(discs[2].getId(), 'Y96eDQZbF4Z26Y5.Sxdbh3wGypo-')

		tracks = release.getTracks()
		self.assertEquals(len(tracks), 12)
		self.assertEquals(tracks[0].getTitle(), 'Crucify')
		self.assertEquals(tracks[4].getTitle(), 'Winter')


	def testReleaseRelations(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Highway_61_Revisited_1.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )
		self.assertEquals(release.getId(),
			makeId('d61a2bd9-81ac-4023-bd22-1c884d4a176c'))

		(rel1, rel2) = release.getRelations(Relation.TO_URL)

		self.assertEquals(rel1.getTargetId(),
			'http://en.wikipedia.org/wiki/Highway_61_Revisited')
		self.assertEquals(rel1.getDirection(), Relation.DIR_NONE)
		self.assertEquals(rel2.getTargetId(),
			'http://www.amazon.com/gp/product/B0000024SI')


	def testVariousArtistsRelease(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Mission_Impossible_2.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )

		artistId = release.getArtist().getId()
		self.assertEquals(artistId, VARIOUS_ARTISTS_ID)

		events = release.getReleaseEventsAsDict()
		self.assertEquals(len(events), 1)
		self.assertEquals(events['EU'], '2000')

		track14 = release.getTracks()[14]
		self.assertEquals(track14.getTitle(), 'Carnival')
		self.assertEquals(track14.getArtist().getName(), 'Tori Amos')


	def testIncompleteReleaseEvent(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Under_the_Pink_1.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )
		self.assertEquals(release.getTitle(), 'Under the Pink')

		events = release.getReleaseEvents()
		self.assertEquals(len(events), 1)
		self.assertEquals(events[0].getDate(), '1994-01-28')


	def testReleaseEvents(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Under_the_Pink_3.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf( release is None )
		self.assertEquals(release.getTitle(), 'Under the Pink')

		events = release.getReleaseEvents()
		self.assertEquals(len(events), 1)
		e1 = events[0]
		self.assertEquals(e1.date, '1994-01-31')
		self.assertEquals(e1.catalogNumber, '82567-2')
		self.assertEquals(e1.barcode, '07567825672')
		self.assertEquals(e1.format, ReleaseEvent.FORMAT_CD)

		self.failIf( e1.label is None )
		self.assertEquals(e1.label.name, 'Atlantic Records')

	def testReleaseGroup(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Under_the_Pink_2.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()

		self.failIf(release is None)
		self.assertEquals(release.getTitle(), 'Under the Pink')

		releaseGroup = release.getReleaseGroup()
		self.failIf(releaseGroup is None)
		self.assertEquals(releaseGroup.id[-36:],
			'ef2b891f-ca73-3e14-b38b-a68699dab8c4')
		self.assertEquals(releaseGroup.getTitle(), 'Under the Pink')
		self.assertEquals(releaseGroup.getType(), NS_MMD_1 + 'Album')


	def testTags(self):
		f = os.path.join(VALID_RELEASE_DIR, 'Highway_61_Revisited_2.xml')
		md = MbXmlParser().parse(f)
		release = md.getRelease()
		
		self.failIf( release is None )
		self.assertEquals(release.getTag('rock').count, 100)
		self.assertEquals(release.getTag('blues rock').count, 40)
		self.assertEquals(release.getTag('folk rock').count, 40)
		self.assertEquals(release.getTag('dylan').count, 4)
		

	def testResultAttributes(self):
		f = os.path.join(VALID_RELEASE_DIR, 'search_result_1.xml')
		md = MbXmlParser().parse(f)

		self.assertEquals(md.releaseResultsOffset, 0)
		self.assertEquals(md.releaseResultsCount, 234)


# EOF



#! /usr/bin/env python
#
# Retrieving an artist's relations to other artists and URLs.
#
# Usage:
#	python getrelations.py artist-id
#
# Interesting Artist IDs for testing:
#	http://musicbrainz.org/artist/ea4dfa26-f633-4da6-a52a-f49ea4897b58
#	http://musicbrainz.org/artist/172e1f1a-504d-4488-b053-6344ba63e6d0
#	http://musicbrainz.org/artist/c0b2500e-0cef-4130-869d-732b23ed9df5
#
# $Id: getrelations.py 7496 2006-05-09 16:52:40Z matt $
#
import sys
import logging
import musicbrainz2.webservice as ws
import musicbrainz2.model as m

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


if len(sys.argv) < 2:
	print "Usage: getrelations.py artist-id"
	sys.exit(1)

q = ws.Query()

try:
	# The result should include all relations to other artists and also
	# relations to URLs.
	#
	inc = ws.ArtistIncludes(artistRelations=True, releaseRelations=True,
		urlRelations=True)
	artist = q.getArtistById(sys.argv[1], inc)
except ws.WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


print "Id         :", artist.id
print "Name       :", artist.name
print


#
# Get the artist's relations to URLs (m.Relation.TO_URL) having the relation
# type 'http://musicbrainz.org/ns/rel-1.0#Wikipedia'. Note that there could
# be more than one relation per type. We just print the first one.
#
urls = artist.getRelationTargets(m.Relation.TO_URL, m.NS_REL_1+'Wikipedia')
if len(urls) > 0:
	print 'Wikipedia:', urls[0]
	print


#
# List discography pages for an artist.
#
for rel in artist.getRelations(m.Relation.TO_URL, m.NS_REL_1+'Discography'):
	print 'Discography:', rel.targetId
	print

#
# If the artist is a group, list all members.
#
if artist.type == m.Artist.TYPE_GROUP:
	allMembers = artist.getRelations(m.Relation.TO_ARTIST,
			m.NS_REL_1+'MemberOfBand')

	uri = m.NS_REL_1+'Additional' 
	coreMembers = [r for r in allMembers if uri not in r.attributes]
	additionalMembers = [r for r in allMembers if uri in r.attributes]

	print 'Group members:'
	for rel in coreMembers:
		start = rel.beginDate or 'foundation'
		end = rel.endDate or 'end'
		print '\t%s (%s to %s)' % (rel.target.name, start, end)

	print
	print 'Additional members:'
	for rel in additionalMembers:
		print '\t', rel.target.name


#
# List all releases for which this artist has acted as the producer.
#
releases = artist.getRelationTargets(m.Relation.TO_RELEASE,
		m.NS_REL_1+'Producer')
print
print 'Credited as producer for:'
for r in releases:
	print '\t', r.title

# EOF



#! /usr/bin/env python
#
# Retrieve a label by ID.
#
# Usage:
#	python getlabel.py label-id
#
# $Id: getlabel.py 9316 2007-08-11 07:38:14Z matt $
#
import sys
import logging
import musicbrainz2.webservice as ws
import musicbrainz2.model as m

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


if len(sys.argv) < 2:
	print "Usage: getlabel.py label-id"
	sys.exit(1)

q = ws.Query()

try:
	# The result should include all aliases.
	#
	inc = ws.LabelIncludes(aliases=True)
	label = q.getLabelById(sys.argv[1], inc)
except ws.WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


print "Id         :", label.id
print "Name       :", label.name
print "SortName   :", label.sortName
print "UniqueName :", label.getUniqueName()
print "Type       :", label.type
print "BeginDate  :", label.beginDate
print "EndDate    :", label.endDate
print "Country    :", label.country
print "Label-Code :", label.code
print "Aliases    :"

for alias in label.aliases:
	print "            ", label.value

# EOF



"""Tests for various model classes."""
import unittest
from musicbrainz2.model import Artist, Release, Track, Relation, Tag, NS_REL_1

class MiscModelTest(unittest.TestCase):
	
	def __init__(self, name):
		unittest.TestCase.__init__(self, name)

	def testAddRelation(self):
		rel = Relation(NS_REL_1+'Producer', Relation.TO_RELEASE, 'a_id',			attributes=[NS_REL_1+'Co'])
		artist = Artist('ar_id', 'Tori Amos', 'Person')
		artist.addRelation(rel)

		rel2 = artist.getRelations(Relation.TO_RELEASE)[0]
		self.assertEquals(rel.getType(), rel2.getType())
		self.assertEquals(rel.getTargetType(), rel2.getTargetType())
		self.assertEquals(rel.getTargetId(), rel2.getTargetId())
		self.assertEquals(rel.getAttributes(), rel2.getAttributes())
		self.assertEquals(rel.getBeginDate(), rel2.getBeginDate())
		self.assertEquals(rel.getEndDate(), rel2.getEndDate())

		self.assertEquals(artist.getRelationTargetTypes(),
			[ Relation.TO_RELEASE ])

		# works because we only have one relation
		self.assertEquals(artist.getRelations(),
			artist.getRelations(Relation.TO_RELEASE))

		rel3 = artist.getRelations(Relation.TO_RELEASE,
			NS_REL_1 + 'Producer')
		self.assertEquals(len(rel3), 1)

		rel4 = artist.getRelations(Relation.TO_RELEASE,
			NS_REL_1 + 'Producer', [NS_REL_1 + 'Co'])
		self.assertEquals(len(rel4), 1)

		rel5 = artist.getRelations(Relation.TO_RELEASE,
			NS_REL_1 + 'Producer', [NS_REL_1 + 'NotThere'])
		self.assertEquals(len(rel5), 0)

		rel6 = artist.getRelations(Relation.TO_RELEASE,
			NS_REL_1 + 'Producer', [NS_REL_1 + 'Co'], 'none')
		self.assertEquals(len(rel6), 1)

		rel6 = artist.getRelations(Relation.TO_RELEASE,
			NS_REL_1 + 'Producer', [NS_REL_1 + 'Co'], 'forward')
		self.assertEquals(len(rel6), 0)


	def testTrackDuration(self):
		t = Track()
		self.assert_( t.getDuration() is None )
		self.assert_( t.getDurationSplit() == (0, 0) )
		t.setDuration(0)
		self.assert_( t.getDurationSplit() == (0, 0) )
		t.setDuration(218666)
		self.assert_( t.getDurationSplit() == (3, 39) )

	def testReleaseIsSingleArtist(self):
		r = Release()
		r.setArtist(Artist(id_=1))
		r.addTrack(Track())
		r.addTrack(Track())
		self.assert_( r.isSingleArtistRelease() )

		r.getTracks()[0].setArtist(Artist(id_=7))
		r.getTracks()[1].setArtist(Artist(id_=8))
		self.assert_( r.isSingleArtistRelease() == False )

		r.getTracks()[0].setArtist(Artist(id_=1))
		r.getTracks()[1].setArtist(Artist(id_=1))
		self.assert_( r.isSingleArtistRelease() )

	def testTags(self):
		a = Artist()
		a.addTag(Tag('foo', 1))
		a.addTag(Tag('bar', 2))
		a.addTag(Tag('bar', 5))

		self.assertEquals(len(a.tags), 2)
		self.assertEquals(a.getTag('foo').count, 1)
		self.assertEquals(a.getTag('bar').count, 7)


class TagTest(unittest.TestCase):
	
	def test__str__(self):
		self.assertEquals("foo", str(Tag(u"foo")))
		self.assertRaises(UnicodeEncodeError, str, Tag(u"f\u014do"))
	
	def test__unicode__(self):
		self.assertEquals(u"foo", unicode(Tag(u"foo")))
		self.assertEquals(u"f\u014do", unicode(Tag(u"f\u014do")))

class ArtistTest(unittest.TestCase):

    def testAddRelease(self):
        release = Release()
        artist = Artist()
        artist.addRelease(release)
        self.assertEquals(artist.releases, [release])

# EOF



#! /usr/bin/env python
#
# Search for a label by name.
#
# Usage:
#	python findlabel.py 'label-name'
#
# $Id: findlabel.py 9316 2007-08-11 07:38:14Z matt $
#
import sys
import logging
from musicbrainz2.webservice import Query, LabelFilter, WebServiceError

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
	

if len(sys.argv) < 2:
	print "Usage: findlabel.py 'label name'"
	sys.exit(1)

q = Query()

try:
	# Search for all labels matching the given name. Limit the results
	# to the 5 best matches. The offset parameter could be used to page
	# through the results.
	#
	f = LabelFilter(name=sys.argv[1], limit=5)
	labelResults = q.getLabels(f)
except WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


# No error occurred, so display the results of the search. It consists of
# LabelResult objects, where each contains a label.
#
for result in labelResults:
	label = result.label
	print "Score     :", result.score
	print "Id        :", label.id
	print "Name      :", label.name
	print "Sort Name :", label.sortName
	print

#
# Now that you have label IDs, you can request a label directly to get more
# detail. See 'getlabel.py' for an example on how to do that.
#

# EOF



#! /usr/bin/env python
#
# This shows the web service interaction needed for a typical CD ripper.
#
# Usage:
#	python ripper.py
#
# $Id: ripper.py 201 2006-03-27 14:43:13Z matt $
#
import sys
import logging
import musicbrainz2.disc as mbdisc
import musicbrainz2.webservice as mbws


# Activate logging.
#
logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


# Setup a Query object.
#
service = mbws.WebService()
query = mbws.Query(service)

# Read the disc in the drive
#
try:
	disc = mbdisc.readDisc()
except mbdisc.DiscError, e:
	print "Error:", e
	sys.exit(1)


# Query for all discs matching the given DiscID.
#
try:
	filter = mbws.ReleaseFilter(discId=disc.getId())
	results = query.getReleases(filter)
except mbws.WebServiceError, e:
	print "Error:", e
	sys.exit(2)


# No disc matching this DiscID has been found.
#
if len(results) == 0:
	print "Disc is not yet in the MusicBrainz database."
	print "Consider adding it via", mbdisc.getSubmissionUrl(disc)
	sys.exit(0)


# Display the returned results to the user.
#
print 'Matching releases:'

for result in results:
	release = result.release
	print 'Artist  :', release.artist.name
	print 'Title   :', release.title
	print


# Select one of the returned releases. We just pick the first one.
#
selectedRelease = results[0].release


# The returned release object only contains title and artist, but no tracks.
# Query the web service once again to get all data we need.
#
try:
	inc = mbws.ReleaseIncludes(artist=True, tracks=True, releaseEvents=True)
	release = query.getReleaseById(selectedRelease.getId(), inc)
except mbws.WebServiceError, e:
	print "Error:", e
	sys.exit(2)


# Now display the returned data.
#
isSingleArtist = release.isSingleArtistRelease()

print "%s - %s" % (release.artist.getUniqueName(), release.title)

i = 1
for t in release.tracks:
	if isSingleArtist:
		title = t.title
	else:
		title = t.artist.name + ' - ' +  t.title

	(minutes, seconds) = t.getDurationSplit()
	print " %2d. %s (%d:%02d)" % (i, title, minutes, seconds)
	i+=1


# All data has been retrieved, now actually rip the CD :-)
#

# EOF



#! /usr/bin/env python
#
# Display data about a MusicBrainz user (user name and password required).
#
# Usage:
#	python user.py
#
# $Id: getuser.py 201 2006-03-27 14:43:13Z matt $
#
import sys
import logging
import getpass
from musicbrainz2.webservice import WebService, WebServiceError, Query

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


user = raw_input('User name: ')
passwd = getpass.getpass('Password: ')

try:
	ws = WebService(host='musicbrainz.org', port=80,
		username=user, password=passwd)
	q = Query(ws)

	user = q.getUserByName(user)

except WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


print 'Name            :', user.name
print 'ShowNag         :', user.showNag
print 'Types           :', ' '.join(user.types)

# EOF



"""Utilities for working with Audio CDs.

This module contains utilities for working with Audio CDs.

The functions in this module need both a working ctypes package (already
included in python-2.5) and an installed libdiscid. If you don't have
libdiscid, it can't be loaded, or your platform isn't supported by either
ctypes or this module, a C{NotImplementedError} is raised when using the
L{readDisc()} function.

@author: Matthias Friedrich <matt@mafr.de>
"""
__revision__ = '$Id: disc.py 11987 2009-08-22 11:57:51Z matt $'

import sys
import urllib
import urlparse
import ctypes
import ctypes.util
from musicbrainz2.model import Disc

__all__ = [ 'DiscError', 'readDisc', 'getSubmissionUrl' ]


class DiscError(IOError):
	"""The Audio CD could not be read.

	This may be simply because no disc was in the drive, the device name
	was wrong or the disc can't be read. Reading errors can occur in case
	of a damaged disc or a copy protection mechanism, for example.
	"""
	pass


def _openLibrary():
	"""Tries to open libdiscid.

	@return: a C{ctypes.CDLL} object, representing the opened library

	@raise NotImplementedError: if the library can't be opened
	"""
	# This only works for ctypes >= 0.9.9.3. Any libdiscid is found,
	# no matter how it's called on this platform.
	try:
		if hasattr(ctypes.cdll, 'find'):
			libDiscId = ctypes.cdll.find('discid')
			_setPrototypes(libDiscId)
			return libDiscId
	except OSError, e:
		raise NotImplementedError('Error opening library: ' + str(e))

	# Try to find the library using ctypes.util
	libName = ctypes.util.find_library('discid')
	if libName != None:
		try:
			libDiscId = ctypes.cdll.LoadLibrary(libName)
			_setPrototypes(libDiscId)
			return libDiscId
		except OSError, e:
			raise NotImplementedError('Error opening library: ' +
				str(e))

	# For compatibility with ctypes < 0.9.9.3 try to figure out the library
	# name without the help of ctypes. We use cdll.LoadLibrary() below,
	# which isn't available for ctypes == 0.9.9.3.
	#
	if sys.platform == 'linux2':
		libName = 'libdiscid.so.0'
	elif sys.platform == 'darwin':
		libName = 'libdiscid.0.dylib'
	elif sys.platform == 'win32':
		libName = 'discid.dll'
	else:
		# This should at least work for Un*x-style operating systems
		libName = 'libdiscid.so.0'

	try:
		libDiscId = ctypes.cdll.LoadLibrary(libName)
		_setPrototypes(libDiscId)
		return libDiscId
	except OSError, e:
		raise NotImplementedError('Error opening library: ' + str(e))

	assert False # not reached


def _setPrototypes(libDiscId):
	ct = ctypes
	libDiscId.discid_new.argtypes = ( )
	libDiscId.discid_new.restype = ct.c_void_p

	libDiscId.discid_free.argtypes = (ct.c_void_p, )

	libDiscId.discid_read.argtypes = (ct.c_void_p, ct.c_char_p)

	libDiscId.discid_get_error_msg.argtypes = (ct.c_void_p, )
	libDiscId.discid_get_error_msg.restype = ct.c_char_p

	libDiscId.discid_get_id.argtypes = (ct.c_void_p, )
	libDiscId.discid_get_id.restype = ct.c_char_p

	libDiscId.discid_get_first_track_num.argtypes = (ct.c_void_p, )
	libDiscId.discid_get_first_track_num.restype = ct.c_int

	libDiscId.discid_get_last_track_num.argtypes = (ct.c_void_p, )
	libDiscId.discid_get_last_track_num.restype = ct.c_int

	libDiscId.discid_get_sectors.argtypes = (ct.c_void_p, )
	libDiscId.discid_get_sectors.restype = ct.c_int

	libDiscId.discid_get_track_offset.argtypes = (ct.c_void_p, ct.c_int)
	libDiscId.discid_get_track_offset.restype = ct.c_int

	libDiscId.discid_get_track_length.argtypes = (ct.c_void_p, ct.c_int)
	libDiscId.discid_get_track_length.restype = ct.c_int


def getSubmissionUrl(disc, host='mm.musicbrainz.org', port=80):
	"""Returns a URL for adding a disc to the MusicBrainz database.

	A fully initialized L{musicbrainz2.model.Disc} object is needed, as
	returned by L{readDisc}. A disc object returned by the web service
	doesn't provide the necessary information.

	Note that the created URL is intended for interactive use and points
	to the MusicBrainz disc submission wizard by default. This method
	just returns a URL, no network connection is needed. The disc drive
	isn't used.

	@param disc: a fully initialized L{musicbrainz2.model.Disc} object
	@param host: a string containing a host name
	@param port: an integer containing a port number

	@return: a string containing the submission URL

	@see: L{readDisc}
	"""
	assert isinstance(disc, Disc), 'musicbrainz2.model.Disc expected'
	discid = disc.getId()
	first = disc.getFirstTrackNum()
	last = disc.getLastTrackNum()
	sectors = disc.getSectors()
	assert None not in (discid, first, last, sectors)

	tracks = last - first + 1
	toc = "%d %d %d " % (first, last, sectors)
	toc = toc + ' '.join( map(lambda x: str(x[0]), disc.getTracks()) )

	query = urllib.urlencode({ 'id': discid, 'toc': toc, 'tracks': tracks })

	if port == 80:
		netloc = host
	else:
		netloc = host + ':' + str(port)

	url = ('http', netloc, '/bare/cdlookup.html', '', query, '')
		
	return urlparse.urlunparse(url)


def readDisc(deviceName=None):
	"""Reads an Audio CD in the disc drive.

	This reads a CD's table of contents (TOC) and calculates the MusicBrainz
	DiscID, which is a 28 character ASCII string. This DiscID can be used
	to retrieve a list of matching releases from the web service (see
	L{musicbrainz2.webservice.Query}).

	Note that an Audio CD has to be in drive for this to work. The
	C{deviceName} argument may be used to set the device. The default
	depends on the operating system (on linux, it's C{'/dev/cdrom'}).
	No network connection is needed for this function.

	If the device doesn't exist or there's no valid Audio CD in the drive,
	a L{DiscError} exception is raised.

	@param deviceName: a string containing the CD drive's device name

	@return: a L{musicbrainz2.model.Disc} object

	@raise DiscError: if there was a problem reading the disc
	@raise NotImplementedError: if DiscID generation isn't supported
	"""
	libDiscId = _openLibrary()

	handle = libDiscId.discid_new()
	assert handle != 0, "libdiscid: discid_new() returned NULL"

	# Access the CD drive. This also works if deviceName is None because
	# ctypes passes a NULL pointer in this case.
	#
	res = libDiscId.discid_read(handle, deviceName)
	if res == 0:
		raise DiscError(libDiscId.discid_get_error_msg(handle))


	# Now extract the data from the result.
	#
	disc = Disc()

	disc.setId( libDiscId.discid_get_id(handle) )

	firstTrackNum = libDiscId.discid_get_first_track_num(handle)
	lastTrackNum = libDiscId.discid_get_last_track_num(handle)

	disc.setSectors(libDiscId.discid_get_sectors(handle))

	for i in range(firstTrackNum, lastTrackNum+1):
		trackOffset = libDiscId.discid_get_track_offset(handle, i)
		trackSectors = libDiscId.discid_get_track_length(handle, i)

		disc.addTrack( (trackOffset, trackSectors) )

	disc.setFirstTrackNum(firstTrackNum)
	disc.setLastTrackNum(lastTrackNum)

	libDiscId.discid_free(handle)

	return disc

# EOF



#! /usr/bin/env python
__revision__ = '$Id: setup.py 12028 2009-09-01 13:15:50Z matt $'

import os
import sys
import os.path
import unittest
from distutils.core import setup, Command

sys.path.insert(0, 'src')
import musicbrainz2

class TestCommand(Command):
	description = 'run all test cases'
	user_options = [ ]
	
	def initialize_options(self): pass

	def finalize_options(self): pass

	def run(self):
		files = [ ]
		for f in os.listdir('test'):
			elems = os.path.splitext(f)
			if f != '__init__.py' and elems[1] == '.py':
				files.append('test.' + elems[0])

		tests = unittest.defaultTestLoader.loadTestsFromNames(files)
		t = unittest.TextTestRunner()
		t.run(tests)


class GenerateDocsCommand(Command):
	description = 'generate the API documentation'
	user_options = [ ]

	def initialize_options(self): pass

	def finalize_options(self): pass

	def run(self):
		from distutils.spawn import find_executable, spawn
		bin = find_executable('epydoc')
		if not bin:
			print>>sys.stderr, 'error: epydoc not found'
			sys.exit(1)
		noPrivate = '--no-private'
		verbose = '-v'
		cmd = (bin, noPrivate, verbose,
			os.path.join('src', 'musicbrainz2'))

		spawn(cmd)


long_description = """\
An interface to the MusicBrainz XML web service
===============================================

python-musicbrainz2 provides simple, object oriented access to the
MusicBrainz web service. It is useful for applications like CD rippers,
taggers, media players, and other tools that need music metadata.

The MusicBrainz Project (see http://musicbrainz.org) collects music
metadata and is maintained by its large and constantly growing user
community.

Most of this package works on python-2.3 and later without further
dependencies. If you want to generate DiscIDs from an audio CD in the
drive, you need ctypes (already included in python-2.5) and libdiscid.
"""

trove_classifiers = [
	'Development Status :: 5 - Production/Stable',
	'Intended Audience :: Developers',
	'License :: OSI Approved :: BSD License',
	'Operating System :: OS Independent',
	'Programming Language :: Python',
	'Topic :: Database :: Front-Ends',
	'Topic :: Multimedia :: Sound/Audio :: CD Audio :: CD Ripping',
	'Topic :: Software Development :: Libraries :: Python Modules',
	'Topic :: Text Processing :: Markup :: XML',
]

setup_args = {
	'name':		'python-musicbrainz2',
	'version':	musicbrainz2.__version__,
	'description':	'An interface to the MusicBrainz XML web service',
	'long_description': long_description,
	'author':	'Matthias Friedrich',
	'author_email':	'matt@mafr.de',
	'url':		'http://musicbrainz.org/products/python-musicbrainz2/',
	'download_url':	'http://ftp.musicbrainz.org/pub/musicbrainz/python-musicbrainz2/',
	'classifiers':	trove_classifiers,
	'license':	'BSD',
	'packages':	[ 'musicbrainz2', 'musicbrainz2.data' ],
	'package_dir':	{ 'musicbrainz2': 'src/musicbrainz2' },
	'scripts':	[ 'bin/mb-submit-disc' ],
	'cmdclass':	{ 'test': TestCommand, 'docs': GenerateDocsCommand },
}

setup(**setup_args)

# EOF



#! /usr/bin/env python
#
# Search for an artist by name.
#
# Usage:
#	python findartist.py 'artist-name'
#
# $Id: findartist.py 8779 2007-01-07 10:01:52Z matt $
#
import sys
import logging
from musicbrainz2.webservice import Query, ArtistFilter, WebServiceError

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
	

if len(sys.argv) < 2:
	print "Usage: findartist.py 'artist name'"
	sys.exit(1)

q = Query()

try:
	# Search for all artists matching the given name. Limit the results
	# to the 5 best matches. The offset parameter could be used to page
	# through the results.
	#
	f = ArtistFilter(name=sys.argv[1], limit=5)
	artistResults = q.getArtists(f)
except WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


# No error occurred, so display the results of the search. It consists of
# ArtistResult objects, where each contains an artist.
#
for result in artistResults:
	artist = result.artist
	print "Score     :", result.score
	print "Id        :", artist.id
	print "Name      :", artist.name
	print "Sort Name :", artist.sortName
	print

#
# Now that you have artist IDs, you can request an artist in more detail, for
# example to display all official albums by that artist. See the 'getartist.py'
# example on how achieve that.
#

# EOF



"""Tests for subclasses of IFilter."""
import unittest
from musicbrainz2.model import Release
from musicbrainz2.webservice import ReleaseFilter


class ReleaseFilterTest(unittest.TestCase):
	def testBasic(self):
		f = ReleaseFilter(title='Under the Pink')
		params = f.createParameters()
		self.assert_( ('title', 'Under the Pink') in params )

	def testReleaseTypes(self):
		f = ReleaseFilter(artistName='Tori Amos', releaseTypes=(
			Release.TYPE_ALBUM, Release.TYPE_OFFICIAL))
		params = f.createParameters()

		self.assertEquals(len(params), 2)
		self.assert_( ('artist', 'Tori Amos') )
		self.assert_( ('releasetypes', 'Album Official') )

	def testQuery(self):
		try:
			f1 = ReleaseFilter(title='Pink', query='xyz')
			self.fail('title and query are mutually exclusive')
		except ValueError:
			pass
		except:
			self.fail('invalid exception')

		# the following shouldn't throw exceptions:
		f2 = ReleaseFilter(title='Pink', limit=10, offset=20)
		f3 = ReleaseFilter(query='Pink', limit=10, offset=20)


# EOF



#! /usr/bin/env python
# 
# Read a CD in the disc drive and calculate a MusicBrainz DiscID.
#
# Usage:
#	python discid.py
#
# $Id: discid.py 201 2006-03-27 14:43:13Z matt $
#
import sys
from musicbrainz2.disc import readDisc, getSubmissionUrl, DiscError

try:
	# Read the disc in the default disc drive. If necessary, you can pass
	# the 'deviceName' parameter to select a different drive.
	#
	disc = readDisc()
except DiscError, e:
	print "DiscID calculation failed:", str(e)
	sys.exit(1)

print 'DiscID      :', disc.id
print 'First Track :', disc.firstTrackNum
print 'Last Track  :', disc.lastTrackNum
print 'Length      :', disc.sectors, 'sectors'

i = disc.firstTrackNum
for (offset, length) in disc.tracks:
	print "Track %-2d    : %8d %8d" % (i, offset, length)
	i += 1

print 'Submit via  :', getSubmissionUrl(disc)

# EOF



# -*- coding: utf-8 -*-

__revision__ = '$Id: releasetypenames.py 8728 2006-12-17 23:42:30Z luks $'

releaseTypeNames = {
	u'http://musicbrainz.org/ns/mmd-1.0#None': u'None',
	u'http://musicbrainz.org/ns/mmd-1.0#Album': u'Album',
	u'http://musicbrainz.org/ns/mmd-1.0#Single': u'Single',
	u'http://musicbrainz.org/ns/mmd-1.0#EP': u'EP',
	u'http://musicbrainz.org/ns/mmd-1.0#Compilation': u'Compilation',
	u'http://musicbrainz.org/ns/mmd-1.0#Soundtrack': u'Soundtrack',
	u'http://musicbrainz.org/ns/mmd-1.0#Spokenword': u'Spokenword',
	u'http://musicbrainz.org/ns/mmd-1.0#Interview': u'Interview',
	u'http://musicbrainz.org/ns/mmd-1.0#Audiobook': u'Audiobook',
	u'http://musicbrainz.org/ns/mmd-1.0#Live': u'Live',
	u'http://musicbrainz.org/ns/mmd-1.0#Remix': u'Remix',
	u'http://musicbrainz.org/ns/mmd-1.0#Other': u'Other',
	u'http://musicbrainz.org/ns/mmd-1.0#Official': u'Official',
	u'http://musicbrainz.org/ns/mmd-1.0#Promotion': u'Promotion',
	u'http://musicbrainz.org/ns/mmd-1.0#Bootleg': u'Bootleg',
	u'http://musicbrainz.org/ns/mmd-1.0#Pseudo-Release': u'Pseudo-Release',
}

# EOF



#! /usr/bin/env python
#
# Retrieve a release by ID and display it.
#
# Usage:
#	python getrelease.py release-id
#
# Interesting releases IDs for testing:
#	http://musicbrainz.org/release/290e10c5-7efc-4f60-ba2c-0dfc0208fbf5
#	http://musicbrainz.org/release/fa9f1bdd-495f-41b9-8944-1a766da29120
#
# $Id: getrelease.py 11853 2009-07-21 09:26:50Z luks $
#
import sys
import logging
import musicbrainz2.webservice as ws
import musicbrainz2.utils as u

logging.basicConfig()
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


if len(sys.argv) < 2:
	print "Usage: getrelease.py release-id"
	sys.exit(1)

q = ws.Query()

try:
	# Additionally to the release itself, we want the server to include
	# the release's artist, all release events, associated discs and
	# the track list.
	#
	inc = ws.ReleaseIncludes(artist=True, releaseEvents=True, labels=True,
			discs=True, tracks=True, releaseGroup=True)
	release = q.getReleaseById(sys.argv[1], inc)
except ws.WebServiceError, e:
	print 'Error:', e
	sys.exit(1)


print "Id          :", release.id
print "Title       :", release.title
print "ASIN        :", release.asin
print "Lang/Script :", release.textLanguage, '/', release.textScript


# Print the main artist of this release.
#
if release.artist:
	print
	print "Artist:"
	print "  Id        :", release.artist.id
	print "  Name      :", release.artist.name
	print "  SortName  :", release.artist.sortName

if release.releaseGroup:
	print
	print "Release Group:"
	print "  Id        :", release.releaseGroup.id
	print "  Title     :", release.releaseGroup.title
	print "  Type      :", release.releaseGroup.type

# Release events are the dates and times when a release took place.
# We also have the catalog numbers and barcodes for some releases.
#
if len(release.releaseEvents) > 0:
	print
	print "Released (earliest: %s):" % release.getEarliestReleaseDate()

for event in release.releaseEvents:
	print "  %s %s" % (u.getCountryName(event.country), event.date),

	if event.catalogNumber:
		print '#' + event.catalogNumber,

	if event.barcode:
		print 'EAN=' + event.barcode,

	if event.label:
		print '(' + event.label.name + ')',

	print


if len(release.discs) > 0:
	print
	print "Discs:"

for disc in release.discs:
	print "  DiscId: %s (%d sectors)" % (disc.id, disc.sectors)


if len(release.tracks) > 0:
	print
	print "Tracks:"

for track in release.tracks:
	print "  Id        :", track.id
	print "  Title     :", track.title
	print "  Duration  :", track.duration
	print

# EOF



# -*- coding: utf-8 -*-

__revision__ = '$Id: countrynames.py 7386 2006-04-30 11:12:55Z matt $'

countryNames = {
	u'BD': u'Bangladesh',
	u'BE': u'Belgium',
	u'BF': u'Burkina Faso',
	u'BG': u'Bulgaria',
	u'BB': u'Barbados',
	u'WF': u'Wallis and Futuna Islands',
	u'BM': u'Bermuda',
	u'BN': u'Brunei Darussalam',
	u'BO': u'Bolivia',
	u'BH': u'Bahrain',
	u'BI': u'Burundi',
	u'BJ': u'Benin',
	u'BT': u'Bhutan',
	u'JM': u'Jamaica',
	u'BV': u'Bouvet Island',
	u'BW': u'Botswana',
	u'WS': u'Samoa',
	u'BR': u'Brazil',
	u'BS': u'Bahamas',
	u'BY': u'Belarus',
	u'BZ': u'Belize',
	u'RU': u'Russian Federation',
	u'RW': u'Rwanda',
	u'RE': u'Reunion',
	u'TM': u'Turkmenistan',
	u'TJ': u'Tajikistan',
	u'RO': u'Romania',
	u'TK': u'Tokelau',
	u'GW': u'Guinea-Bissau',
	u'GU': u'Guam',
	u'GT': u'Guatemala',
	u'GR': u'Greece',
	u'GQ': u'Equatorial Guinea',
	u'GP': u'Guadeloupe',
	u'JP': u'Japan',
	u'GY': u'Guyana',
	u'GF': u'French Guiana',
	u'GE': u'Georgia',
	u'GD': u'Grenada',
	u'GB': u'United Kingdom',
	u'GA': u'Gabon',
	u'SV': u'El Salvador',
	u'GN': u'Guinea',
	u'GM': u'Gambia',
	u'GL': u'Greenland',
	u'GI': u'Gibraltar',
	u'GH': u'Ghana',
	u'OM': u'Oman',
	u'TN': u'Tunisia',
	u'JO': u'Jordan',
	u'HT': u'Haiti',
	u'HU': u'Hungary',
	u'HK': u'Hong Kong',
	u'HN': u'Honduras',
	u'HM': u'Heard and Mc Donald Islands',
	u'VE': u'Venezuela',
	u'PR': u'Puerto Rico',
	u'PW': u'Palau',
	u'PT': u'Portugal',
	u'SJ': u'Svalbard and Jan Mayen Islands',
	u'PY': u'Paraguay',
	u'IQ': u'Iraq',
	u'PA': u'Panama',
	u'PF': u'French Polynesia',
	u'PG': u'Papua New Guinea',
	u'PE': u'Peru',
	u'PK': u'Pakistan',
	u'PH': u'Philippines',
	u'PN': u'Pitcairn',
	u'PL': u'Poland',
	u'PM': u'St. Pierre and Miquelon',
	u'ZM': u'Zambia',
	u'EH': u'Western Sahara',
	u'EE': u'Estonia',
	u'EG': u'Egypt',
	u'ZA': u'South Africa',
	u'EC': u'Ecuador',
	u'IT': u'Italy',
	u'VN': u'Viet Nam',
	u'SB': u'Solomon Islands',
	u'ET': u'Ethiopia',
	u'SO': u'Somalia',
	u'ZW': u'Zimbabwe',
	u'SA': u'Saudi Arabia',
	u'ES': u'Spain',
	u'ER': u'Eritrea',
	u'MD': u'Moldova, Republic of',
	u'MG': u'Madagascar',
	u'MA': u'Morocco',
	u'MC': u'Monaco',
	u'UZ': u'Uzbekistan',
	u'MM': u'Myanmar',
	u'ML': u'Mali',
	u'MO': u'Macau',
	u'MN': u'Mongolia',
	u'MH': u'Marshall Islands',
	u'MK': u'Macedonia, The Former Yugoslav Republic of',
	u'MU': u'Mauritius',
	u'MT': u'Malta',
	u'MW': u'Malawi',
	u'MV': u'Maldives',
	u'MQ': u'Martinique',
	u'MP': u'Northern Mariana Islands',
	u'MS': u'Montserrat',
	u'MR': u'Mauritania',
	u'UG': u'Uganda',
	u'MY': u'Malaysia',
	u'MX': u'Mexico',
	u'IL': u'Israel',
	u'FR': u'France',
	u'IO': u'British Indian Ocean Territory',
	u'SH': u'St. Helena',
	u'FI': u'Finland',
	u'FJ': u'Fiji',
	u'FK': u'Falkland Islands (Malvinas)',
	u'FM': u'Micronesia, Federated States of',
	u'FO': u'Faroe Islands',
	u'NI': u'Nicaragua',
	u'NL': u'Netherlands',
	u'NO': u'Norway',
	u'NA': u'Namibia',
	u'VU': u'Vanuatu',
	u'NC': u'New Caledonia',
	u'NE': u'Niger',
	u'NF': u'Norfolk Island',
	u'NG': u'Nigeria',
	u'NZ': u'New Zealand',
	u'ZR': u'Zaire',
	u'NP': u'Nepal',
	u'NR': u'Nauru',
	u'NU': u'Niue',
	u'CK': u'Cook Islands',
	u'CI': u'Cote d\'Ivoire',
	u'CH': u'Switzerland',
	u'CO': u'Colombia',
	u'CN': u'China',
	u'CM': u'Cameroon',
	u'CL': u'Chile',
	u'CC': u'Cocos (Keeling) Islands',
	u'CA': u'Canada',
	u'CG': u'Congo',
	u'CF': u'Central African Republic',
	u'CZ': u'Czech Republic',
	u'CY': u'Cyprus',
	u'CX': u'Christmas Island',
	u'CR': u'Costa Rica',
	u'CV': u'Cape Verde',
	u'CU': u'Cuba',
	u'SZ': u'Swaziland',
	u'SY': u'Syrian Arab Republic',
	u'KG': u'Kyrgyzstan',
	u'KE': u'Kenya',
	u'SR': u'Suriname',
	u'KI': u'Kiribati',
	u'KH': u'Cambodia',
	u'KN': u'Saint Kitts and Nevis',
	u'KM': u'Comoros',
	u'ST': u'Sao Tome and Principe',
	u'SI': u'Slovenia',
	u'KW': u'Kuwait',
	u'SN': u'Senegal',
	u'SM': u'San Marino',
	u'SL': u'Sierra Leone',
	u'SC': u'Seychelles',
	u'KZ': u'Kazakhstan',
	u'KY': u'Cayman Islands',
	u'SG': u'Singapore',
	u'SE': u'Sweden',
	u'SD': u'Sudan',
	u'DO': u'Dominican Republic',
	u'DM': u'Dominica',
	u'DJ': u'Djibouti',
	u'DK': u'Denmark',
	u'VG': u'Virgin Islands (British)',
	u'DE': u'Germany',
	u'YE': u'Yemen',
	u'DZ': u'Algeria',
	u'US': u'United States',
	u'UY': u'Uruguay',
	u'YT': u'Mayotte',
	u'UM': u'United States Minor Outlying Islands',
	u'LB': u'Lebanon',
	u'LC': u'Saint Lucia',
	u'LA': u'Lao People\'s Democratic Republic',
	u'TV': u'Tuvalu',
	u'TW': u'Taiwan',
	u'TT': u'Trinidad and Tobago',
	u'TR': u'Turkey',
	u'LK': u'Sri Lanka',
	u'LI': u'Liechtenstein',
	u'LV': u'Latvia',
	u'TO': u'Tonga',
	u'LT': u'Lithuania',
	u'LU': u'Luxembourg',
	u'LR': u'Liberia',
	u'LS': u'Lesotho',
	u'TH': u'Thailand',
	u'TF': u'French Southern Territories',
	u'TG': u'Togo',
	u'TD': u'Chad',
	u'TC': u'Turks and Caicos Islands',
	u'LY': u'Libyan Arab Jamahiriya',
	u'VA': u'Vatican City State (Holy See)',
	u'VC': u'Saint Vincent and The Grenadines',
	u'AE': u'United Arab Emirates',
	u'AD': u'Andorra',
	u'AG': u'Antigua and Barbuda',
	u'AF': u'Afghanistan',
	u'AI': u'Anguilla',
	u'VI': u'Virgin Islands (U.S.)',
	u'IS': u'Iceland',
	u'IR': u'Iran (Islamic Republic of)',
	u'AM': u'Armenia',
	u'AL': u'Albania',
	u'AO': u'Angola',
	u'AN': u'Netherlands Antilles',
	u'AQ': u'Antarctica',
	u'AS': u'American Samoa',
	u'AR': u'Argentina',
	u'AU': u'Australia',
	u'AT': u'Austria',
	u'AW': u'Aruba',
	u'IN': u'India',
	u'TZ': u'Tanzania, United Republic of',
	u'AZ': u'Azerbaijan',
	u'IE': u'Ireland',
	u'ID': u'Indonesia',
	u'UA': u'Ukraine',
	u'QA': u'Qatar',
	u'MZ': u'Mozambique',
	u'BA': u'Bosnia and Herzegovina',
	u'CD': u'Congo, The Democratic Republic of the',
	u'CS': u'Serbia and Montenegro',
	u'HR': u'Croatia',
	u'KP': u'Korea (North), Democratic People\'s Republic of',
	u'KR': u'Korea (South), Republic of',
	u'SK': u'Slovakia',
	u'SU': u'Soviet Union (historical, 1922-1991)',
	u'TL': u'East Timor',
	u'XC': u'Czechoslovakia (historical, 1918-1992)',
	u'XE': u'Europe',
	u'XG': u'East Germany (historical, 1949-1990)',
	u'XU': u'[Unknown Country]',
	u'XW': u'[Worldwide]',
	u'YU': u'Yugoslavia (historical, 1918-1992)',
}

# EOF



"""Classes for interacting with the MusicBrainz XML web service.

The L{WebService} class talks to a server implementing the MusicBrainz XML
web service. It mainly handles URL generation and network I/O. Use this
if maximum control is needed.

The L{Query} class provides a convenient interface to the most commonly
used features of the web service. By default it uses L{WebService} to
retrieve data and the L{XML parser <musicbrainz2.wsxml>} to parse the
responses. The results are object trees using the L{MusicBrainz domain
model <musicbrainz2.model>}.

@author: Matthias Friedrich <matt@mafr.de>
"""
__revision__ = '$Id: webservice.py 13325 2011-11-03 13:39:40Z luks $'

import urllib
import urllib2
import urlparse
import logging
import musicbrainz2
from musicbrainz2.model import Release
from musicbrainz2.wsxml import MbXmlParser, ParseError
import musicbrainz2.utils as mbutils

__all__ = [
	'WebServiceError', 'AuthenticationError', 'ConnectionError',
	'RequestError', 'ResourceNotFoundError', 'ResponseError', 
	'IIncludes', 'ArtistIncludes', 'ReleaseIncludes', 'TrackIncludes',
	'LabelIncludes', 'ReleaseGroupIncludes',
	'IFilter', 'ArtistFilter', 'ReleaseFilter', 'TrackFilter',
	'UserFilter', 'LabelFilter', 'ReleaseGroupFilter',
	'IWebService', 'WebService', 'Query',
]


class IWebService(object):
	"""An interface all concrete web service classes have to implement.

	All web service classes have to implement this and follow the
	method specifications.
	"""

	def get(self, entity, id_, include, filter, version):
		"""Query the web service.

		Using this method, you can either get a resource by id (using
		the C{id_} parameter, or perform a query on all resources of
		a type.

		The C{filter} and the C{id_} parameter exclude each other. If
		you are using a filter, you may not set C{id_} and vice versa.

		Returns a file-like object containing the result or raises a
		L{WebServiceError} or one of its subclasses in case of an
		error. Which one is used depends on the implementing class.

		@param entity: a string containing the entity's name
		@param id_: a string containing a UUID, or the empty string
		@param include: a tuple containing values for the 'inc' parameter
		@param filter: parameters, depending on the entity
		@param version: a string containing the web service version to use

		@return: a file-like object

		@raise WebServiceError: in case of errors
		"""
		raise NotImplementedError()


	def post(self, entity, id_, data, version):
		"""Submit data to the web service.

		@param entity: a string containing the entity's name
		@param id_: a string containing a UUID, or the empty string
		@param data: A string containing the data to post
		@param version: a string containing the web service version to use

		@return: a file-like object

		@raise WebServiceError: in case of errors
		"""
		raise NotImplementedError()


class WebServiceError(Exception):
	"""A web service error has occurred.

	This is the base class for several other web service related
	exceptions.
	"""

	def __init__(self, msg='Webservice Error', reason=None):
		"""Constructor.

		Set C{msg} to an error message which explains why this
		exception was raised. The C{reason} parameter should be the
		original exception which caused this L{WebService} exception
		to be raised. If given, it has to be an instance of
		C{Exception} or one of its child classes.

		@param msg: a string containing an error message
		@param reason: another exception instance, or None
		"""
		Exception.__init__(self)
		self.msg = msg
		self.reason = reason

	def __str__(self):
		"""Makes this class printable.

		@return: a string containing an error message
		"""
		return self.msg


class ConnectionError(WebServiceError):
	"""Getting a server connection failed.

	This exception is mostly used if the client couldn't connect to
	the server because of an invalid host name or port. It doesn't
	make sense if the web service in question doesn't use the network.
	"""
	pass


class RequestError(WebServiceError):
	"""An invalid request was made.

	This exception is raised if the client made an invalid request.
	That could be syntactically invalid identifiers or unknown or
	invalid parameter values.
	"""
	pass


class ResourceNotFoundError(WebServiceError):
	"""No resource with the given ID exists.

	This is usually a wrapper around IOError (which is superclass of
	HTTPError).
	"""
	pass


class AuthenticationError(WebServiceError):
	"""Authentication failed.

	This is thrown if user name, password or realm were invalid while
	trying to access a protected resource.
	"""
	pass


class ResponseError(WebServiceError):
	"""The returned resource was invalid.

	This may be due to a malformed XML document or if the requested
	data wasn't part of the response. It can only occur in case of
	bugs in the web service itself.
	"""
	pass

class DigestAuthHandler(urllib2.HTTPDigestAuthHandler):
	"""Patched DigestAuthHandler to correctly handle Digest Auth according to RFC 2617.
	
	This will allow multiple qop values in the WWW-Authenticate header (e.g. "auth,auth-int").
	The only supported qop value is still auth, though.
	See http://bugs.python.org/issue9714
	
	@author: Kuno Woudt
	"""
	def get_authorization(self, req, chal):
		qop = chal.get('qop')
		if qop and ',' in qop and 'auth' in qop.split(','):
			chal['qop'] = 'auth'
		
		return urllib2.HTTPDigestAuthHandler.get_authorization(self, req, chal)

class WebService(IWebService):
	"""An interface to the MusicBrainz XML web service via HTTP.

	By default, this class uses the MusicBrainz server but may be
	configured for accessing other servers as well using the
	L{constructor <__init__>}. This implements L{IWebService}, so
	additional documentation on method parameters can be found there.
	"""

	def __init__(self, host='musicbrainz.org', port=80, pathPrefix='/ws',
			username=None, password=None, realm='musicbrainz.org',
			opener=None, userAgent=None):
		"""Constructor.

		This can be used without parameters. In this case, the
		MusicBrainz server will be used.

		@param host: a string containing a host name
		@param port: an integer containing a port number
		@param pathPrefix: a string prepended to all URLs
		@param username: a string containing a MusicBrainz user name
		@param password: a string containing the user's password
		@param realm: a string containing the realm used for authentication
		@param opener: an C{urllib2.OpenerDirector} object used for queries
		@param userAgent: a string containing the user agent
		"""
		self._host = host
		self._port = port
		self._username = username
		self._password = password
		self._realm = realm
		self._pathPrefix = pathPrefix
		self._log = logging.getLogger(str(self.__class__))

		if opener is None:
			self._opener = urllib2.build_opener()
		else:
			self._opener = opener

		if userAgent is None:
			self._userAgent = "python-musicbrainz/" + musicbrainz2.__version__
		else:
			self._userAgent = userAgent.replace("-", "/") \
								+ " python-musicbrainz/" \
								+ musicbrainz2.__version__

		passwordMgr = self._RedirectPasswordMgr()
		authHandler = DigestAuthHandler(passwordMgr)
		authHandler.add_password(self._realm, (), # no host set
			self._username, self._password)
		self._opener.add_handler(authHandler)


	def _makeUrl(self, entity, id_, include=( ), filter={ },
			version='1', type_='xml'):
		params = dict(filter)
		if type_ is not None:
			params['type'] = type_
		if len(include) > 0:
			params['inc'] = ' '.join(include)

		netloc = self._host
		if self._port != 80:
			netloc += ':' + str(self._port)
		path = '/'.join((self._pathPrefix, version, entity, id_))

		query = urllib.urlencode(params)

		url = urlparse.urlunparse(('http', netloc, path, '', query,''))

		return url


	def _openUrl(self, url, data=None):
		req = urllib2.Request(url)
		req.add_header('User-Agent', self._userAgent)
		return self._opener.open(req, data)


	def get(self, entity, id_, include=( ), filter={ }, version='1'):
		"""Query the web service via HTTP-GET.

		Returns a file-like object containing the result or raises a
		L{WebServiceError}. Conditions leading to errors may be
		invalid entities, IDs, C{include} or C{filter} parameters
		and unsupported version numbers.

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid IDs or parameters
		@raise AuthenticationError: invalid user name and/or password
		@raise ResourceNotFoundError: resource doesn't exist

		@see: L{IWebService.get}
		"""
		url = self._makeUrl(entity, id_, include, filter, version)

		self._log.debug('GET ' + url)

		try:
			return self._openUrl(url)
		except urllib2.HTTPError, e:
			self._log.debug("GET failed: " + str(e))
			if e.code == 400:   # in python 2.4: httplib.BAD_REQUEST
				raise RequestError(str(e), e)
			elif e.code == 401: # httplib.UNAUTHORIZED
				raise AuthenticationError(str(e), e)
			elif e.code == 404: # httplib.NOT_FOUND
				raise ResourceNotFoundError(str(e), e)
			else:
				raise WebServiceError(str(e), e)
		except urllib2.URLError, e:
			self._log.debug("GET failed: " + str(e))
			raise ConnectionError(str(e), e)


	def post(self, entity, id_, data, version='1'):
		"""Send data to the web service via HTTP-POST.

		Note that this may require authentication. You can set
		user name, password and realm in the L{constructor <__init__>}.

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid IDs or parameters
		@raise AuthenticationError: invalid user name and/or password
		@raise ResourceNotFoundError: resource doesn't exist

		@see: L{IWebService.post}
		"""
		url = self._makeUrl(entity, id_, version=version, type_=None)

		self._log.debug('POST ' + url)
		self._log.debug('POST-BODY: ' + data)

		try:
			return self._openUrl(url, data)
		except urllib2.HTTPError, e:
			self._log.debug("POST failed: " + str(e))
			if e.code == 400:   # in python 2.4: httplib.BAD_REQUEST
				raise RequestError(str(e), e)
			elif e.code == 401: # httplib.UNAUTHORIZED
				raise AuthenticationError(str(e), e)
			elif e.code == 404: # httplib.NOT_FOUND
				raise ResourceNotFoundError(str(e), e)
			else:
				raise WebServiceError(str(e), e)
		except urllib2.URLError, e:
			self._log.debug("POST failed: " + str(e))
			raise ConnectionError(str(e), e)


	# Special password manager which also works with redirects by simply
	# ignoring the URI. As a consequence, only *ONE* (username, password)
	# tuple per realm can be used for all URIs.
	#
	class _RedirectPasswordMgr(urllib2.HTTPPasswordMgr):
		def __init__(self):
			self._realms = { }

		def find_user_password(self, realm, uri):
			# ignoring the uri parameter intentionally
			try:
				return self._realms[realm]
			except KeyError:
				return (None, None)

		def add_password(self, realm, uri, username, password):
			# ignoring the uri parameter intentionally
			self._realms[realm] = (username, password)


class IFilter(object):
	"""A filter for collections.

	This is the interface all filters have to implement. Filter classes
	are initialized with a set of criteria and are then applied to
	collections of items. The criteria are usually strings or integer
	values, depending on the filter.

	Note that all strings passed to filters should be unicode strings
	(python type C{unicode}). Standard strings are converted to unicode
	internally, but have a limitation: Only 7 Bit pure ASCII characters
	may be used, otherwise a C{UnicodeDecodeError} is raised.
	"""
	def createParameters(self):
		"""Create a list of query parameters.

		This method creates a list of (C{parameter}, C{value}) tuples,
		based on the contents of the implementing subclass.
		C{parameter} is a string containing a parameter name
		and C{value} an arbitrary string. No escaping of those strings
		is required. 

		@return: a sequence of (key, value) pairs
		"""
		raise NotImplementedError()


class ArtistFilter(IFilter):
	"""A filter for the artist collection."""

	def __init__(self, name=None, limit=None, offset=None, query=None):
		"""Constructor.

		The C{query} parameter may contain a query in U{Lucene syntax
		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
		Note that the C{name} and C{query} may not be used together.

		@param name: a unicode string containing the artist's name
		@param limit: the maximum number of artists to return
		@param offset: start results at this zero-based offset
		@param query: a string containing a query in Lucene syntax
		"""
		self._params = [
			('name', name),
			('limit', limit),
			('offset', offset),
			('query', query),
		]

		if not _paramsValid(self._params):
			raise ValueError('invalid combination of parameters')

	def createParameters(self):
		return _createParameters(self._params)


class LabelFilter(IFilter):
	"""A filter for the label collection."""

	def __init__(self, name=None, limit=None, offset=None, query=None):
		"""Constructor.

		The C{query} parameter may contain a query in U{Lucene syntax
		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
		Note that the C{name} and C{query} may not be used together.

		@param name: a unicode string containing the label's name
		@param limit: the maximum number of labels to return
		@param offset: start results at this zero-based offset
		@param query: a string containing a query in Lucene syntax
		"""
		self._params = [
			('name', name),
			('limit', limit),
			('offset', offset),
			('query', query),
		]

		if not _paramsValid(self._params):
			raise ValueError('invalid combination of parameters')

	def createParameters(self):
		return _createParameters(self._params)

class ReleaseGroupFilter(IFilter):
	"""A filter for the release group collection."""

	def __init__(self, title=None, releaseTypes=None, artistName=None,
			artistId=None, limit=None, offset=None, query=None):
		"""Constructor.

		If C{artistId} is set, only releases matching those IDs are
		returned.  The C{releaseTypes} parameter allows you to limit
		the types of the release groups returned. You can set it to
		C{(Release.TYPE_ALBUM, Release.TYPE_OFFICIAL)}, for example,
		to only get officially released albums. Note that those values
		are connected using the I{AND} operator. MusicBrainz' support
		is currently very limited, so C{Release.TYPE_LIVE} and
		C{Release.TYPE_COMPILATION} exclude each other (see U{the
		documentation on release attributes
		<http://wiki.musicbrainz.org/AlbumAttribute>} for more
		information and all valid values).

		If both the C{artistName} and the C{artistId} parameter are
		given, the server will ignore C{artistName}.

		The C{query} parameter may contain a query in U{Lucene syntax
		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
		Note that C{query} may not be used together with the other
		parameters except for C{limit} and C{offset}.

		@param title: a unicode string containing the release group's title
		@param releaseTypes: a sequence of release type URIs
		@param artistName: a unicode string containing the artist's name
		@param artistId: a unicode string containing the artist's ID
		@param limit: the maximum number of release groups to return
		@param offset: start results at this zero-based offset
		@param query: a string containing a query in Lucene syntax

		@see: the constants in L{musicbrainz2.model.Release}
		"""
		if releaseTypes is None or len(releaseTypes) == 0:
			releaseTypesStr = None
		else:
			releaseTypesStr = ' '.join(map(mbutils.extractFragment, releaseTypes))

		self._params = [
			('title', title),
			('releasetypes', releaseTypesStr),
			('artist', artistName),
			('artistid', mbutils.extractUuid(artistId)),
			('limit', limit),
			('offset', offset),
			('query', query),
		]

		if not _paramsValid(self._params):
			raise ValueError('invalid combination of parameters')

	def createParameters(self):
		return _createParameters(self._params)


class ReleaseFilter(IFilter):
	"""A filter for the release collection."""

	def __init__(self, title=None, discId=None, releaseTypes=None,
			artistName=None, artistId=None, limit=None,
			offset=None, query=None, trackCount=None):
		"""Constructor.

		If C{discId} or C{artistId} are set, only releases matching
		those IDs are returned. The C{releaseTypes} parameter allows
		to limit the types of the releases returned. You can set it to
		C{(Release.TYPE_ALBUM, Release.TYPE_OFFICIAL)}, for example,
		to only get officially released albums. Note that those values
		are connected using the I{AND} operator. MusicBrainz' support
		is currently very limited, so C{Release.TYPE_LIVE} and
		C{Release.TYPE_COMPILATION} exclude each other (see U{the
		documentation on release attributes
		<http://wiki.musicbrainz.org/AlbumAttribute>} for more
		information and all valid values).

		If both the C{artistName} and the C{artistId} parameter are
		given, the server will ignore C{artistName}.

		The C{query} parameter may contain a query in U{Lucene syntax
		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
		Note that C{query} may not be used together with the other
		parameters except for C{limit} and C{offset}.

		@param title: a unicode string containing the release's title
		@param discId: a unicode string containing the DiscID
		@param releaseTypes: a sequence of release type URIs
		@param artistName: a unicode string containing the artist's name
		@param artistId: a unicode string containing the artist's ID
		@param limit: the maximum number of releases to return
		@param offset: start results at this zero-based offset
		@param query: a string containing a query in Lucene syntax
		@param trackCount: the number of tracks in the release

		@see: the constants in L{musicbrainz2.model.Release}
		"""
		if releaseTypes is None or len(releaseTypes) == 0:
			releaseTypesStr = None
		else:
			tmp = [ mbutils.extractFragment(x) for x in releaseTypes ]
			releaseTypesStr = ' '.join(tmp)

		self._params = [
			('title', title),
			('discid', discId),
			('releasetypes', releaseTypesStr),
			('artist', artistName),
			('artistid', mbutils.extractUuid(artistId)),
			('limit', limit),
			('offset', offset),
			('query', query),
			('count', trackCount),
		]

		if not _paramsValid(self._params):
			raise ValueError('invalid combination of parameters')

	def createParameters(self):
		return _createParameters(self._params)


class TrackFilter(IFilter):
	"""A filter for the track collection."""

	def __init__(self, title=None, artistName=None, artistId=None,
			releaseTitle=None, releaseId=None,
			duration=None, puid=None, limit=None, offset=None,
			query=None):
		"""Constructor.

		If C{artistId}, C{releaseId} or C{puid} are set, only tracks
		matching those IDs are returned.

		The server will ignore C{artistName} and C{releaseTitle} if
		C{artistId} or ${releaseId} are set respectively.

		The C{query} parameter may contain a query in U{Lucene syntax
		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
		Note that C{query} may not be used together with the other
		parameters except for C{limit} and C{offset}.

		@param title: a unicode string containing the track's title
		@param artistName: a unicode string containing the artist's name
		@param artistId: a string containing the artist's ID
		@param releaseTitle: a unicode string containing the release's title
		@param releaseId: a string containing the release's title
		@param duration: the track's length in milliseconds
		@param puid: a string containing a PUID
		@param limit: the maximum number of releases to return
		@param offset: start results at this zero-based offset
		@param query: a string containing a query in Lucene syntax
		"""
		self._params = [
			('title', title),
			('artist', artistName),
			('artistid', mbutils.extractUuid(artistId)),
			('release', releaseTitle),
			('releaseid', mbutils.extractUuid(releaseId)),
			('duration', duration),
			('puid', puid),
			('limit', limit),
			('offset', offset),
			('query', query),
		]

		if not _paramsValid(self._params):
			raise ValueError('invalid combination of parameters')

	def createParameters(self):
		return _createParameters(self._params)


class UserFilter(IFilter):
	"""A filter for the user collection."""

	def __init__(self, name=None):
		"""Constructor.

		@param name: a unicode string containing a MusicBrainz user name
		"""
		self._name = name

	def createParameters(self):
		if self._name is not None:
			return [ ('name', self._name.encode('utf-8')) ]
		else:
			return [ ]


class IIncludes(object):
	"""An interface implemented by include tag generators."""
	def createIncludeTags(self):
		raise NotImplementedError()


class ArtistIncludes(IIncludes):
	"""A specification on how much data to return with an artist.

	Example:

	>>> from musicbrainz2.model import Release
	>>> from musicbrainz2.webservice import ArtistIncludes
	>>> inc = ArtistIncludes(artistRelations=True, releaseRelations=True,
	... 		releases=(Release.TYPE_ALBUM, Release.TYPE_OFFICIAL))
	>>>

	The MusicBrainz server only supports some combinations of release
	types for the C{releases} and C{vaReleases} include tags. At the
	moment, not more than two release types should be selected, while
	one of them has to be C{Release.TYPE_OFFICIAL},
	C{Release.TYPE_PROMOTION} or C{Release.TYPE_BOOTLEG}.

	@note: Only one of C{releases} and C{vaReleases} may be given.
	"""
	def __init__(self, aliases=False, releases=(), vaReleases=(),
			artistRelations=False, releaseRelations=False,
			trackRelations=False, urlRelations=False, tags=False,
			ratings=False, releaseGroups=False):

		assert not isinstance(releases, basestring)
		assert not isinstance(vaReleases, basestring)
		assert len(releases) == 0 or len(vaReleases) == 0

		self._includes = {
			'aliases':		aliases,
			'artist-rels':		artistRelations,
			'release-groups':	releaseGroups,
			'release-rels':		releaseRelations,
			'track-rels':		trackRelations,
			'url-rels':		urlRelations,
			'tags':			tags,
			'ratings':		ratings,
		}

		for elem in releases:
			self._includes['sa-' + mbutils.extractFragment(elem)] = True

		for elem in vaReleases:
			self._includes['va-' + mbutils.extractFragment(elem)] = True

	def createIncludeTags(self):
		return _createIncludes(self._includes)


class ReleaseIncludes(IIncludes):
	"""A specification on how much data to return with a release."""
	def __init__(self, artist=False, counts=False, releaseEvents=False,
			discs=False, tracks=False,
			artistRelations=False, releaseRelations=False,
			trackRelations=False, urlRelations=False,
			labels=False, tags=False, ratings=False, isrcs=False,
			releaseGroup=False):
		self._includes = {
			'artist':		artist,
			'counts':		counts,
			'labels':		labels,
			'release-groups':	releaseGroup,
			'release-events':	releaseEvents,
			'discs':		discs,
			'tracks':		tracks,
			'artist-rels':		artistRelations,
			'release-rels':		releaseRelations,
			'track-rels':		trackRelations,
			'url-rels':		urlRelations,
			'tags':			tags,
			'ratings':		ratings,
			'isrcs':		isrcs,
		}

		# Requesting labels without releaseEvents makes no sense,
		# so we pull in releaseEvents, if necessary.
		if labels and not releaseEvents:
			self._includes['release-events'] = True
		# Ditto for isrcs with no tracks
		if isrcs and not tracks:
			self._includes['tracks'] = True

	def createIncludeTags(self):
		return _createIncludes(self._includes)


class ReleaseGroupIncludes(IIncludes):
	"""A specification on how much data to return with a release group."""

	def __init__(self, artist=False, releases=False, tags=False):
		"""Constructor.

		@param artist: Whether to include the release group's main artist info.
		@param releases: Whether to include the release group's releases.
		"""
		self._includes = {
			'artist':		artist,
			'releases':		releases,
		}

	def createIncludeTags(self):
		return _createIncludes(self._includes)


class TrackIncludes(IIncludes):
	"""A specification on how much data to return with a track."""
	def __init__(self, artist=False, releases=False, puids=False,
			artistRelations=False, releaseRelations=False,
			trackRelations=False, urlRelations=False, tags=False,
			ratings=False, isrcs=False):
		self._includes = {
			'artist':		artist,
			'releases':		releases,
			'puids':		puids,
			'artist-rels':		artistRelations,
			'release-rels':		releaseRelations,
			'track-rels':		trackRelations,
			'url-rels':		urlRelations,
			'tags':			tags,
			'ratings':		ratings,
			'isrcs':		isrcs,
		}

	def createIncludeTags(self):
		return _createIncludes(self._includes)


class LabelIncludes(IIncludes):
	"""A specification on how much data to return with a label."""
	def __init__(self, aliases=False, tags=False, ratings=False):
		self._includes = {
			'aliases':		aliases,
			'tags':			tags,
			'ratings':		ratings,
		}

	def createIncludeTags(self):
		return _createIncludes(self._includes)


class Query(object):
	"""A simple interface to the MusicBrainz web service.

	This is a facade which provides a simple interface to the MusicBrainz
	web service. It hides all the details like fetching data from a server,
	parsing the XML and creating an object tree. Using this class, you can
	request data by ID or search the I{collection} of all resources
	(artists, releases, or tracks) to retrieve those matching given
	criteria. This document contains examples to get you started.


	Working with Identifiers
	========================

	MusicBrainz uses absolute URIs as identifiers. For example, the artist
	'Tori Amos' is identified using the following URI::
		http://musicbrainz.org/artist/c0b2500e-0cef-4130-869d-732b23ed9df5

	In some situations it is obvious from the context what type of 
	resource an ID refers to. In these cases, abbreviated identifiers may
	be used, which are just the I{UUID} part of the URI. Thus the ID above
	may also be written like this::
		c0b2500e-0cef-4130-869d-732b23ed9df5

	All methods in this class which require IDs accept both the absolute
	URI and the abbreviated form (aka the relative URI).


	Creating a Query Object
	=======================

	In most cases, creating a L{Query} object is as simple as this:

	>>> import musicbrainz2.webservice as ws
	>>> q = ws.Query()
	>>>

	The instantiated object uses the standard L{WebService} class to
	access the MusicBrainz web service. If you want to use a different 
	server or you have to pass user name and password because one of
	your queries requires authentication, you have to create the
	L{WebService} object yourself and configure it appropriately.
	This example uses the MusicBrainz test server and also sets
	authentication data:

	>>> import musicbrainz2.webservice as ws
	>>> service = ws.WebService(host='test.musicbrainz.org',
	...				username='whatever', password='secret')
	>>> q = ws.Query(service)
	>>>


	Querying for Individual Resources
	=================================

	If the MusicBrainz ID of a resource is known, then the L{getArtistById},
	L{getReleaseById}, or L{getTrackById} method can be used to retrieve
	it. Example:

	>>> import musicbrainz2.webservice as ws
	>>> q = ws.Query()
	>>> artist = q.getArtistById('c0b2500e-0cef-4130-869d-732b23ed9df5')
	>>> artist.name
	u'Tori Amos'
	>>> artist.sortName
	u'Amos, Tori'
	>>> print artist.type
	http://musicbrainz.org/ns/mmd-1.0#Person
	>>>

	This returned just the basic artist data, however. To get more detail
	about a resource, the C{include} parameters may be used which expect
	an L{ArtistIncludes}, L{ReleaseIncludes}, or L{TrackIncludes} object,
	depending on the resource type.

	To get data about a release which also includes the main artist
	and all tracks, for example, the following query can be used:

	>>> import musicbrainz2.webservice as ws
	>>> q = ws.Query()
	>>> releaseId = '33dbcf02-25b9-4a35-bdb7-729455f33ad7'
	>>> include = ws.ReleaseIncludes(artist=True, tracks=True)
	>>> release = q.getReleaseById(releaseId, include)
	>>> release.title
	u'Tales of a Librarian'
	>>> release.artist.name
	u'Tori Amos'
	>>> release.tracks[0].title
	u'Precious Things'
	>>>

	Note that the query gets more expensive for the server the more
	data you request, so please be nice.


	Searching in Collections
	========================

	For each resource type (artist, release, and track), there is one
	collection which contains all resources of a type. You can search
	these collections using the L{getArtists}, L{getReleases}, and
	L{getTracks} methods. The collections are huge, so you have to
	use filters (L{ArtistFilter}, L{ReleaseFilter}, or L{TrackFilter})
	to retrieve only resources matching given criteria.

	For example, If you want to search the release collection for
	releases with a specified DiscID, you would use L{getReleases}
	and a L{ReleaseFilter} object:

	>>> import musicbrainz2.webservice as ws
	>>> q = ws.Query()
	>>> filter = ws.ReleaseFilter(discId='8jJklE258v6GofIqDIrE.c5ejBE-')
	>>> results = q.getReleases(filter=filter)
	>>> results[0].score
	100
	>>> results[0].release.title
	u'Under the Pink'
	>>>

	The query returns a list of results (L{wsxml.ReleaseResult} objects
	in this case), which are ordered by score, with a higher score
	indicating a better match. Note that those results don't contain
	all the data about a resource. If you need more detail, you can then
	use the L{getArtistById}, L{getReleaseById}, or L{getTrackById}
	methods to request the resource.

	All filters support the C{limit} argument to limit the number of
	results returned. This defaults to 25, but the server won't send
	more than 100 results to save bandwidth and processing power. Using
	C{limit} and the C{offset} parameter, you can page through the
	results.


	Error Handling
	==============

	All methods in this class raise a L{WebServiceError} exception in case
	of errors. Depending on the method, a subclass of L{WebServiceError} may
	be raised which allows an application to handle errors more precisely.
	The following example handles connection errors (invalid host name
	etc.) separately and all other web service errors in a combined
	catch clause:

	>>> try:
	...     artist = q.getArtistById('c0b2500e-0cef-4130-869d-732b23ed9df5')
	... except ws.ConnectionError, e:
	...     pass # implement your error handling here
	... except ws.WebServiceError, e:
	...     pass # catches all other web service errors
	... 
	>>>
	"""

	def __init__(self, ws=None, wsFactory=WebService, clientId=None):
		"""Constructor.

		The C{ws} parameter has to be a subclass of L{IWebService}.
		If it isn't given, the C{wsFactory} parameter is used to
		create an L{IWebService} subclass.

		If the constructor is called without arguments, an instance
		of L{WebService} is used, preconfigured to use the MusicBrainz
		server. This should be enough for most users.

		If you want to use queries which require authentication you
		have to pass a L{WebService} instance where user name and
		password have been set.

		The C{clientId} parameter is required for data submission.
		The format is C{'application-version'}, where C{application}
		is your application's name and C{version} is a version
		number which may not include a '-' character.
		Even if you don't plan to submit data, setting this parameter is
		encouraged because it will set the user agent used to make requests if
		you don't supply the C{ws} parameter.

		@param ws: a subclass instance of L{IWebService}, or None
		@param wsFactory: a callable object which creates an object
		@param clientId: a unicode string containing the application's ID
		"""
		if ws is None:
			self._ws = wsFactory(userAgent=clientId)
		else:
			self._ws = ws

		self._clientId = clientId
		self._log = logging.getLogger(str(self.__class__))


	def getArtistById(self, id_, include=None):
		"""Returns an artist.

		If no artist with that ID can be found, C{include} contains
		invalid tags or there's a server problem, an exception is
		raised.

		@param id_: a string containing the artist's ID
		@param include: an L{ArtistIncludes} object, or None

		@return: an L{Artist <musicbrainz2.model.Artist>} object, or None

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResourceNotFoundError: artist doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		uuid = mbutils.extractUuid(id_, 'artist')
		result = self._getFromWebService('artist', uuid, include)
		artist = result.getArtist()
		if artist is not None:
			return artist
		else:
			raise ResponseError("server didn't return artist")


	def getArtists(self, filter):
		"""Returns artists matching given criteria.

		@param filter: an L{ArtistFilter} object

		@return: a list of L{musicbrainz2.wsxml.ArtistResult} objects

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResponseError: server returned invalid data
		"""
		result = self._getFromWebService('artist', '', filter=filter)
		return result.getArtistResults()

	def getLabelById(self, id_, include=None):
		"""Returns a L{model.Label}
		
		If no label with that ID can be found, or there is a server problem,
		an exception is raised.
		
		@param id_: a string containing the label's ID.

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResourceNotFoundError: release doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		uuid = mbutils.extractUuid(id_, 'label')
		result = self._getFromWebService('label', uuid, include)
		label = result.getLabel()
		if label is not None:
			return label
		else:
			raise ResponseError("server didn't return a label")
	
	def getLabels(self, filter):
		result = self._getFromWebService('label', '', filter=filter)
		return result.getLabelResults()

	def getReleaseById(self, id_, include=None):
		"""Returns a release.

		If no release with that ID can be found, C{include} contains
		invalid tags or there's a server problem, and exception is
		raised.

		@param id_: a string containing the release's ID
		@param include: a L{ReleaseIncludes} object, or None

		@return: a L{Release <musicbrainz2.model.Release>} object, or None

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResourceNotFoundError: release doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		uuid = mbutils.extractUuid(id_, 'release')
		result = self._getFromWebService('release', uuid, include)
		release = result.getRelease()
		if release is not None:
			return release
		else:
			raise ResponseError("server didn't return release")


	def getReleases(self, filter):
		"""Returns releases matching given criteria.

		@param filter: a L{ReleaseFilter} object

		@return: a list of L{musicbrainz2.wsxml.ReleaseResult} objects

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResponseError: server returned invalid data
		"""
		result = self._getFromWebService('release', '', filter=filter)
		return result.getReleaseResults()
	
	def getReleaseGroupById(self, id_, include=None):
		"""Returns a release group.

		If no release group with that ID can be found, C{include}
		contains invalid tags, or there's a server problem, an
		exception is raised.

		@param id_: a string containing the release group's ID
		@param include: a L{ReleaseGroupIncludes} object, or None

		@return: a L{ReleaseGroup <musicbrainz2.model.ReleaseGroup>} object, or None

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResourceNotFoundError: release doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		uuid = mbutils.extractUuid(id_, 'release-group')
		result = self._getFromWebService('release-group', uuid, include)
		releaseGroup = result.getReleaseGroup()
		if releaseGroup is not None:
			return releaseGroup
		else:
			raise ResponseError("server didn't return releaseGroup")

	def getReleaseGroups(self, filter):
		"""Returns release groups matching the given criteria.
		
		@param filter: a L{ReleaseGroupFilter} object
		
		@return: a list of L{musicbrainz2.wsxml.ReleaseGroupResult} objects
		
		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResponseError: server returned invalid data
		"""
		result = self._getFromWebService('release-group', '', filter=filter)
		return result.getReleaseGroupResults()

	def getTrackById(self, id_, include=None):
		"""Returns a track.

		If no track with that ID can be found, C{include} contains
		invalid tags or there's a server problem, an exception is
		raised.

		@param id_: a string containing the track's ID
		@param include: a L{TrackIncludes} object, or None

		@return: a L{Track <musicbrainz2.model.Track>} object, or None

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResourceNotFoundError: track doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		uuid = mbutils.extractUuid(id_, 'track')
		result = self._getFromWebService('track', uuid, include)
		track = result.getTrack()
		if track is not None:
			return track
		else:
			raise ResponseError("server didn't return track")


	def getTracks(self, filter):
		"""Returns tracks matching given criteria.

		@param filter: a L{TrackFilter} object

		@return: a list of L{musicbrainz2.wsxml.TrackResult} objects

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise ResponseError: server returned invalid data
		"""
		result = self._getFromWebService('track', '', filter=filter)
		return result.getTrackResults()


	def getUserByName(self, name):
		"""Returns information about a MusicBrainz user.

		You can only request user data if you know the user name and
		password for that account. If username and/or password are
		incorrect, an L{AuthenticationError} is raised.

		See the example in L{Query} on how to supply user name and
		password.

		@param name: a unicode string containing the user's name

		@return: a L{User <musicbrainz2.model.User>} object

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or include tags
		@raise AuthenticationError: invalid user name and/or password
		@raise ResourceNotFoundError: track doesn't exist
		@raise ResponseError: server returned invalid data
		"""
		filter = UserFilter(name=name)
		result = self._getFromWebService('user', '', None, filter)

		if len(result.getUserList()) > 0:
			return result.getUserList()[0]
		else:
			raise ResponseError("response didn't contain user data")


	def _getFromWebService(self, entity, id_, include=None, filter=None):
		if filter is None:
			filterParams = [ ]
		else:
			filterParams = filter.createParameters()

		if include is None:
			includeParams = [ ]
		else:
			includeParams = include.createIncludeTags()

		stream = self._ws.get(entity, id_, includeParams, filterParams)
		try:
			parser = MbXmlParser()
			return parser.parse(stream)
		except ParseError, e:
			raise ResponseError(str(e), e)


	def submitPuids(self, tracks2puids):
		"""Submit track to PUID mappings.

		The C{tracks2puids} parameter has to be a dictionary, with the
		keys being MusicBrainz track IDs (either as absolute URIs or
		in their 36 character ASCII representation) and the values
		being PUIDs (ASCII, 36 characters).

		Note that this method only works if a valid user name and
		password have been set. See the example in L{Query} on how
		to supply authentication data.

		@param tracks2puids: a dictionary mapping track IDs to PUIDs

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid track or PUIDs
		@raise AuthenticationError: invalid user name and/or password
		"""
		assert self._clientId is not None, 'Please supply a client ID'
		params = [ ]
		params.append( ('client', self._clientId.encode('utf-8')) )

		for (trackId, puid) in tracks2puids.iteritems():
			trackId = mbutils.extractUuid(trackId, 'track')
			params.append( ('puid', trackId + ' ' + puid) )

		encodedStr = urllib.urlencode(params, True)

		self._ws.post('track', '', encodedStr)
	
	def submitISRCs(self, tracks2isrcs):
		"""Submit track to ISRC mappings.

		The C{tracks2isrcs} parameter has to be a dictionary, with the
		keys being MusicBrainz track IDs (either as absolute URIs or
		in their 36 character ASCII representation) and the values
		being ISRCs (ASCII, 12 characters).

		Note that this method only works if a valid user name and
		password have been set. See the example in L{Query} on how
		to supply authentication data.

		@param tracks2isrcs: a dictionary mapping track IDs to ISRCs

		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid track or ISRCs
		@raise AuthenticationError: invalid user name and/or password
		"""
		params = [ ]

		for (trackId, isrc) in tracks2isrcs.iteritems():
			trackId = mbutils.extractUuid(trackId, 'track')
			params.append( ('isrc', trackId + ' ' + isrc) )

		encodedStr = urllib.urlencode(params, True)

		self._ws.post('track', '', encodedStr)

	def addToUserCollection(self, releases):
		"""Add releases to a user's collection.

		The releases parameter must be a list. It can contain either L{Release}
		objects or a string representing a MusicBrainz release ID (either as
		absolute URIs or in their 36 character ASCII representation).

		Adding a release that is already in the collection has no effect.

		@param releases: a list of releases to add to the user collection

		@raise ConnectionError: couldn't connect to server
		@raise AuthenticationError: invalid user name and/or password
		"""
		rels = []
		for release in releases:
			if isinstance(release, Release):
				rels.append(mbutils.extractUuid(release.id))
			else:
				rels.append(mbutils.extractUuid(release))
		encodedStr = urllib.urlencode({'add': ",".join(rels)}, True)
		self._ws.post('collection', '', encodedStr)

	def removeFromUserCollection(self, releases):
		"""Remove releases from a user's collection.

		The releases parameter must be a list. It can contain either L{Release}
		objects or a string representing a MusicBrainz release ID (either as
		absolute URIs or in their 36 character ASCII representation).

		Removing a release that is not in the collection has no effect.

		@param releases: a list of releases to remove from the user collection

		@raise ConnectionError: couldn't connect to server
		@raise AuthenticationError: invalid user name and/or password
		"""
		rels = []
		for release in releases:
			if isinstance(release, Release):
				rels.append(mbutils.extractUuid(release.id))
			else:
				rels.append(mbutils.extractUuid(release))
		encodedStr = urllib.urlencode({'remove': ",".join(rels)}, True)
		self._ws.post('collection', '', encodedStr)

	def getUserCollection(self, offset=0, maxitems=100):
		"""Get the releases that are in a user's collection
		
		A maximum of 100 items will be returned for any one call
		to this method. To fetch more than 100 items, use the offset
		parameter.

		@param offset: the offset to start fetching results from
		@param maxitems: the upper limit on items to return

		@return: a list of L{musicbrainz2.wsxml.ReleaseResult} objects

		@raise ConnectionError: couldn't connect to server
		@raise AuthenticationError: invalid user name and/or password
		"""
		params = { 'offset': offset, 'maxitems': maxitems }
		
		stream = self._ws.get('collection', '', filter=params)
		try:
			parser = MbXmlParser()
			result = parser.parse(stream)
		except ParseError, e:
			raise ResponseError(str(e), e)
		
		return result.getReleaseResults()

	def submitUserTags(self, entityUri, tags):
		"""Submit folksonomy tags for an entity.

		Note that all previously existing tags from the authenticated
		user are replaced with the ones given to this method. Other
		users' tags are not affected.
		
		@param entityUri: a string containing an absolute MB ID
		@param tags: A list of either L{Tag <musicbrainz2.model.Tag>} objects
		             or strings

		@raise ValueError: invalid entityUri
		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID, entity or tags
		@raise AuthenticationError: invalid user name and/or password
		"""
		entity = mbutils.extractEntityType(entityUri)
		uuid = mbutils.extractUuid(entityUri, entity)
		params = (
			('type', 'xml'),
			('entity', entity),
			('id', uuid),
			('tags', ','.join([unicode(tag).encode('utf-8') for tag in tags]))
		)

		encodedStr = urllib.urlencode(params)

		self._ws.post('tag', '', encodedStr)


	def getUserTags(self, entityUri):
		"""Returns a list of folksonomy tags a user has applied to an entity.

		The given parameter has to be a fully qualified MusicBrainz ID, as
		returned by other library functions.
		
		Note that this method only works if a valid user name and
		password have been set. Only the tags the authenticated user
		applied to the entity will be returned. If username and/or
		password are incorrect, an AuthenticationError is raised.
		
		This method will return a list of L{Tag <musicbrainz2.model.Tag>}
		objects.
		
		@param entityUri: a string containing an absolute MB ID
		
		@raise ValueError: invalid entityUri
  		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or entity
		@raise AuthenticationError: invalid user name and/or password
		"""
		entity = mbutils.extractEntityType(entityUri)
		uuid = mbutils.extractUuid(entityUri, entity)
		params = { 'entity': entity, 'id': uuid }
		
		stream = self._ws.get('tag', '', filter=params)
		try:
			parser = MbXmlParser()
			result = parser.parse(stream)
		except ParseError, e:
			raise ResponseError(str(e), e)
		
		return result.getTagList()

	def submitUserRating(self, entityUri, rating):
		"""Submit rating for an entity.

		Note that all previously existing rating from the authenticated
		user are replaced with the one given to this method. Other
		users' ratings are not affected.
		
		@param entityUri: a string containing an absolute MB ID
		@param rating: A L{Rating <musicbrainz2.model.Rating>} object
		             or integer

		@raise ValueError: invalid entityUri
		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID, entity or tags
		@raise AuthenticationError: invalid user name and/or password
		"""
		entity = mbutils.extractEntityType(entityUri)
		uuid = mbutils.extractUuid(entityUri, entity)
		params = (
			('type', 'xml'),
			('entity', entity),
			('id', uuid),
			('rating', unicode(rating).encode('utf-8'))
		)

		encodedStr = urllib.urlencode(params)

		self._ws.post('rating', '', encodedStr)


	def getUserRating(self, entityUri):
		"""Return the rating a user has applied to an entity.

		The given parameter has to be a fully qualified MusicBrainz
		ID, as returned by other library functions.
		
		Note that this method only works if a valid user name and
		password have been set. Only the rating the authenticated user
		applied to the entity will be returned. If username and/or
		password are incorrect, an AuthenticationError is raised.
		
		This method will return a L{Rating <musicbrainz2.model.Rating>}
		object.
		
		@param entityUri: a string containing an absolute MB ID
		
		@raise ValueError: invalid entityUri
  		@raise ConnectionError: couldn't connect to server
		@raise RequestError: invalid ID or entity
		@raise AuthenticationError: invalid user name and/or password
		"""
		entity = mbutils.extractEntityType(entityUri)
		uuid = mbutils.extractUuid(entityUri, entity)
		params = { 'entity': entity, 'id': uuid }
		
		stream = self._ws.get('rating', '', filter=params)
		try:
			parser = MbXmlParser()
			result = parser.parse(stream)
		except ParseError, e:
			raise ResponseError(str(e), e)
		
		return result.getRating()

	def submitCDStub(self, cdstub):
		"""Submit a CD Stub to the database.

		The number of tracks added to the CD Stub must match the TOC and DiscID
		otherwise the submission wil fail. The submission will also fail if 
		the Disc ID is already in the MusicBrainz database.

		This method will only work if no user name and password are set.

		@param cdstub: a L{CDStub} object to submit
		
		@raise RequestError: Missmatching TOC/Track information or the
		       the CD Stub already exists or the Disc ID already exists
		"""
		assert self._clientId is not None, 'Please supply a client ID'
		disc = cdstub._disc
		params = [ ]
		params.append( ('client', self._clientId.encode('utf-8')) )
		params.append( ('discid', disc.id) )
		params.append( ('title', cdstub.title) )
		params.append( ('artist', cdstub.artist) )
		if cdstub.barcode != "":
			params.append( ('barcode', cdstub.barcode) )
		if cdstub.comment != "":
			params.append( ('comment', cdstub.comment) )

		trackind = 0
		for track,artist in cdstub.tracks:
			params.append( ('track%d' % trackind, track) )
			if artist != "":
				params.append( ('artist%d' % trackind, artist) )

			trackind += 1

		toc = "%d %d %d " % (disc.firstTrackNum, disc.lastTrackNum, disc.sectors)
	        toc = toc + ' '.join( map(lambda x: str(x[0]), disc.getTracks()) )

		params.append( ('toc', toc) )

		encodedStr = urllib.urlencode(params)
		self._ws.post('release', '', encodedStr)

def _createIncludes(tagMap):
	selected = filter(lambda x: x[1] == True, tagMap.items())
	return map(lambda x: x[0], selected)

def _createParameters(params):
	"""Remove (x, None) tuples and encode (x, str/unicode) to utf-8."""
	ret = [ ]
	for p in params:
		if isinstance(p[1], (str, unicode)):
			ret.append( (p[0], p[1].encode('utf-8')) )
		elif p[1] is not None:
			ret.append(p)

	return ret

def _paramsValid(params):
	"""Check if the query parameter collides with other parameters."""
	tmp = [ ]
	for name, value in params:
		if value is not None and name not in ('offset', 'limit'):
			tmp.append(name)

	if 'query' in tmp and len(tmp) > 1:
		return False
	else:
		return True

if __name__ == '__main__':
	import doctest
	doctest.testmod()

# EOF
